@@ -18,6 +18,12 @@ brew install libpng
 # Assume zlib is always available.
 # brew install zlib 
 
+# Install Blas and Lapack.
+# brew install openblas
+# brew install lapack
+# brew ls --verbose openblas
+# brew ls --verbose lapack
+
 # Install SeqAn 2.4.0
 brew install brewsci/bio/seqan@2
 
@@ -36,7 +42,7 @@ tar -xvf spoa-v3.4.0.tar.gz
 # Build the static library.
 mkdir build-static
 cd build-static
-cmake ../spoa-v3.4.0 -DBUILD_SHARED_LIBS=OFF
+cmake ../spoa-v3.4.0 -DBUILD_SHARED_LIBS=OFF -Dspoa_optimize_for_native=OFF
 make -j all
 make install
 cd 
