@@ -4,7 +4,11 @@
 // Shasta.
 #include "Assembler.hpp"
 #include "AlignmentGraph.hpp"
+#include "Align4.hpp"
+#include "AssemblerOptions.hpp"
 #include "compressAlignment.hpp"
+#include "Reads.hpp"
+#include "span.hpp"
 #include "timestamp.hpp"
 using namespace shasta;
 
@@ -197,52 +201,10 @@ void Assembler::alignOverlappingOrientedReads(
 
 
 // Compute an alignment for each alignment candidate.
-// Store summary information for the ones that are good enough,
-// without storing details of the alignment.
+// Store the alignments the satisfy our criteria.
 void Assembler::computeAlignments(
 
-    // Alignment method.
-    int alignmentMethod,
-
-    // Marker frequency threshold.
-    // When computing an alignment between two oriented reads,
-    // marker kmers that appear more than this number of times
-    // in either of the two oriented reads are discarded
-    // (in both oriented reads).
-    uint32_t maxMarkerFrequency,
-
-    // The maximum ordinal skip to be tolerated between successive markers
-    // in the alignment.
-    size_t maxSkip,
-
-    // The maximum relative ordinal drift to be tolerated between successive markers
-    // in the alignment.
-    size_t maxDrift,
-
-    // Minimum number of alignment markers for an alignment to be used.
-    size_t minAlignedMarkerCount,
-
-    // The minimum fraction of aligned markers for an alignment to be used.
-    double minAlignedFraction,
-
-    // Maximum left/right trim (in bases) for an alignment to be used.
-    size_t maxTrim,
-
-    // Scores used to compute method 1 and 3 alignments.
-    int matchScore,
-    int mismatchScore,
-    int gapScore,
-
-    // Parameters for alignment method 3.
-    double downsamplingFactor,
-    int bandExtend,
-    int maxBand,
-
-    // If true, discard containment alignments.
-    bool suppressContainments,
-
-    // If true, store good alignments in a compressed format.
-    bool storeAlignments,
+    const AlignOptions& alignOptions,
 
     // Number of threads. If zero, a number of threads equal to
     // the number of virtual processors is used.
@@ -261,29 +223,21 @@ void Assembler::computeAlignments(
 
     // Store parameters so they are accessible to the threads.
     auto& data = computeAlignmentsData;
-    data.alignmentMethod = alignmentMethod;
-    data.maxMarkerFrequency = maxMarkerFrequency;
-    data.maxSkip = maxSkip;
-    data.maxDrift = maxDrift;
-    data.minAlignedMarkerCount = minAlignedMarkerCount;
-    data.minAlignedFraction = minAlignedFraction;
-    data.maxTrim = maxTrim;
-    data.matchScore = matchScore;
-    data.mismatchScore = mismatchScore;
-    data.gapScore = gapScore;
-    data.downsamplingFactor = downsamplingFactor;
-    data.bandExtend = bandExtend;
-    data.maxBand = maxBand;
-    data.suppressContainments = suppressContainments;
-    data.storeAlignments = storeAlignments;
+    data.alignOptions = &alignOptions;
 
     // Adjust the numbers of threads, if necessary.
     if(threadCount == 0) {
         threadCount = std::thread::hardware_concurrency();
     }
 
+    // For alignment method 4, compute sorted markers.
+    if(alignOptions.alignMethod == 4) {
+        cout << timestamp << "Computing sorted markers." << endl;
+        computeSortedMarkers(threadCount);
+    }
+
     // Pick the batch size for computing alignments.
-    size_t batchSize = 10000;
+    size_t batchSize = 10;
     if(batchSize > alignmentCandidates.candidates.size()/threadCount) {
         batchSize = alignmentCandidates.candidates.size()/threadCount;
     }
@@ -301,13 +255,10 @@ void Assembler::computeAlignments(
     runThreads(&Assembler::computeAlignmentsThreadFunction, threadCount);
     cout << timestamp << "Alignment computation completed." << endl;
 
-    // Store alignmentInfos found by each thread in the global alignmentInfos.
-    cout << timestamp << "Storing the alignment info objects." << endl;
+    // Store the alignments found by each thread.
+    cout << timestamp << "Storing the alignment found by each thread." << endl;
     alignmentData.createNew(largeDataName("AlignmentData"), largeDataPageSize);
-    
-    if (data.storeAlignments) {
-        compressedAlignments.createNew(largeDataName("CompressedAlignments"), largeDataPageSize);
-    }
+    compressedAlignments.createNew(largeDataName("CompressedAlignments"), largeDataPageSize);
     
     for(size_t threadId=0; threadId<threadCount; threadId++) {
         const vector<AlignmentData>& threadAlignmentData = data.threadAlignmentData[threadId];
@@ -315,25 +266,28 @@ void Assembler::computeAlignments(
             alignmentData.push_back(ad);
         }
 
-        if (data.storeAlignments) {
-            const auto threadCompressedAlignments = data.threadCompressedAlignments[threadId];
-            const auto size = threadCompressedAlignments->size();
-            for(size_t i=0; i<size; i++) {
-                compressedAlignments.appendVector(
-                    (*threadCompressedAlignments)[i].begin(),
-                    (*threadCompressedAlignments)[i].end()
-                );
-            }
-
-            // Clean up thread storage.
-            data.threadCompressedAlignments[threadId]->remove();
+        const auto threadCompressedAlignments = data.threadCompressedAlignments[threadId];
+        const auto size = threadCompressedAlignments->size();
+        for(size_t i=0; i<size; i++) {
+            compressedAlignments.appendVector(
+                (*threadCompressedAlignments)[i].begin(),
+                (*threadCompressedAlignments)[i].end()
+            );
         }
+
+        // Clean up thread storage.
+        data.threadCompressedAlignments[threadId]->remove();
     }
 
     // Release unused allocated memory.
     alignmentData.unreserve();
     compressedAlignments.unreserve();
 
+    // For alignment method 4, remove the sorted markers.
+    if(alignOptions.alignMethod == 4) {
+        sortedMarkers.remove();
+    }
+
     cout << "Found and stored " << alignmentData.size() << " good alignments." << endl;
     cout << timestamp << "Creating alignment table." << endl;
     computeAlignmentTable();
@@ -344,11 +298,6 @@ void Assembler::computeAlignments(
     cout << "completed in " << tTotal << " s." << endl;
 
     cout << timestamp;
-    if (data.storeAlignments) {
-        cout << "Storing compressed alignments for potential reuse." << endl;
-    } else {
-        cout << "Not storing compressed alignments." << endl;
-    }
 }
 
 
@@ -365,21 +314,43 @@ void Assembler::computeAlignmentsThreadFunction(size_t threadId)
 
     const bool debug = false;
     auto& data = computeAlignmentsData;
-    const size_t alignmentMethod = data.alignmentMethod;
-    const uint32_t maxMarkerFrequency = data.maxMarkerFrequency;
-    const size_t maxSkip = data.maxSkip;
-    const size_t maxDrift = data.maxDrift;
-    const size_t minAlignedMarkerCount = data.minAlignedMarkerCount;
-    const double minAlignedFraction = data.minAlignedFraction;
-    const size_t maxTrim = data.maxTrim;
-    const int matchScore = data.matchScore;
-    const int mismatchScore = data.mismatchScore;
-    const int gapScore = data.gapScore;
-    const double downsamplingFactor = data.downsamplingFactor;
-    const int bandExtend = data.bandExtend;
-    const int maxBand = data.maxBand;
-    const bool suppressContainments = data.suppressContainments;
-    const bool storeAlignments = data.storeAlignments;
+    const size_t alignmentMethod = data.alignOptions->alignMethod;
+    const uint32_t maxMarkerFrequency = data.alignOptions->maxMarkerFrequency;
+    const size_t maxSkip = data.alignOptions->maxSkip;
+    const size_t maxDrift = data.alignOptions->maxDrift;
+    const size_t minAlignedMarkerCount = data.alignOptions->minAlignedMarkerCount;
+    const double minAlignedFraction = data.alignOptions->minAlignedFraction;
+    const size_t maxTrim = data.alignOptions->maxTrim;
+    const int matchScore = data.alignOptions->matchScore;
+    const int mismatchScore = data.alignOptions->mismatchScore;
+    const int gapScore = data.alignOptions->gapScore;
+    const double downsamplingFactor = data.alignOptions->downsamplingFactor;
+    const int bandExtend = data.alignOptions->bandExtend;
+    const int maxBand = data.alignOptions->maxBand;
+    const bool suppressContainments = data.alignOptions->suppressContainments;
+
+
+    // Align4-specific items.
+    Align4::Options align4Options;
+    MemoryMapped::ByteAllocator byteAllocator;
+    if(alignmentMethod == 4) {
+        align4Options.deltaX = data.alignOptions->align4DeltaX;
+        align4Options.deltaY = data.alignOptions->align4DeltaY;
+        align4Options.minEntryCountPerCell = data.alignOptions->align4MinEntryCountPerCell;
+        align4Options.maxDistanceFromBoundary = data.alignOptions->align4MaxDistanceFromBoundary;
+        align4Options.minAlignedMarkerCount = minAlignedMarkerCount;
+        align4Options.minAlignedFraction = minAlignedFraction;
+        align4Options.maxSkip = maxSkip;
+        align4Options.maxDrift = maxDrift;
+        align4Options.maxTrim = maxTrim;
+        align4Options.maxBand = maxBand;
+        align4Options.matchScore = matchScore;
+        align4Options.mismatchScore = mismatchScore;
+        align4Options.gapScore = gapScore;
+        byteAllocator.createNew(
+            largeDataName("tmp-ByteAllocator-" + to_string(threadId)),
+            largeDataPageSize, 2ULL * 1024 * 1024 * 1024);
+    }
 
     vector<AlignmentData>& threadAlignmentData = data.threadAlignmentData[threadId];
     
@@ -387,12 +358,9 @@ void Assembler::computeAlignmentsThreadFunction(size_t threadId)
         make_shared< MemoryMapped::VectorOfVectors<char, uint64_t> >();
     data.threadCompressedAlignments[threadId] = thisThreadCompressedAlignmentsPointer;
     auto& thisThreadCompressedAlignments = *thisThreadCompressedAlignmentsPointer;
-
-    if (storeAlignments) {
-        thisThreadCompressedAlignments.createNew(
-            largeDataName("tmp-ThreadGlobalCompressedAlignments-" + to_string(threadId)),
-            largeDataPageSize);
-    }
+    thisThreadCompressedAlignments.createNew(
+        largeDataName("tmp-ThreadGlobalCompressedAlignments-" + to_string(threadId)),
+        largeDataPageSize);
 
     uint64_t begin, end;
     while(getNextBatch(begin, end)) {
@@ -435,6 +403,13 @@ void Assembler::computeAlignmentsThreadFunction(size_t threadId)
                         matchScore, mismatchScore, gapScore,
                         downsamplingFactor, bandExtend, maxBand,
                         alignment, alignmentInfo);
+                } else if(alignmentMethod == 4) {
+                    alignOrientedReads4(orientedReadIds[0], orientedReadIds[1],
+                        align4Options,
+                        byteAllocator,
+                        alignment, alignmentInfo,
+                        false);
+                    SHASTA_ASSERT(byteAllocator.isEmpty());
                 } else {
                     SHASTA_ASSERT(0);
                 }
@@ -497,18 +472,22 @@ void Assembler::computeAlignmentsThreadFunction(size_t threadId)
             // cout << orientedReadIds[0] << " " << orientedReadIds[1] << " good." << endl;
             threadAlignmentData.push_back(AlignmentData(candidate, alignmentInfo));
 
-            // Store the compressed alignment if so configured.
-            if (storeAlignments) {
-                shasta::compress(alignment, compressedAlignment);
-
-                thisThreadCompressedAlignments.appendVector(
-                    compressedAlignment.c_str(),
-                    compressedAlignment.c_str() + compressedAlignment.size()
-                );
-            }
+            // Store the alignment in compressed form.
+            shasta::compress(alignment, compressedAlignment);
+            thisThreadCompressedAlignments.appendVector(
+                compressedAlignment.c_str(),
+                compressedAlignment.c_str() + compressedAlignment.size()
+            );
         }
     }
 
+    if(alignmentMethod == 4) {
+        std::lock_guard<std::mutex> lock(mutex);
+        cout << "Thread " << threadId << " byte allocator: " <<
+            byteAllocator.getMaxAllocatedByteCount() << "/" <<
+            2ULL * 1024 * 1024 * 1024 << endl;
+    }
+
     thisThreadCompressedAlignments.unreserve();
 }
 
@@ -595,6 +574,11 @@ void Assembler::accessAlignmentData()
     alignmentData.accessExistingReadOnly(largeDataName("AlignmentData"));
     alignmentTable.accessExistingReadOnly(largeDataName("AlignmentTable"));
 }
+void Assembler::accessAlignmentDataReadWrite()
+{
+    alignmentData.accessExistingReadWrite(largeDataName("AlignmentData"));
+    alignmentTable.accessExistingReadWrite(largeDataName("AlignmentTable"));
+}
 
 
 
@@ -612,7 +596,9 @@ void Assembler::checkAlignmentDataAreOpen() const
 // orientation (this may involve a swap and/or reverse complement
 // of the AlignmentInfo stored in the alignmentTable).
 vector< pair<OrientedReadId, shasta::AlignmentInfo> >
-    Assembler::findOrientedAlignments(OrientedReadId orientedReadId0Argument) const
+    Assembler::findOrientedAlignments(
+        OrientedReadId orientedReadId0Argument,
+        bool inReadGraphOnly) const
 {
     const ReadId readId0 = orientedReadId0Argument.getReadId();
     const ReadId strand0 = orientedReadId0Argument.getStrand();
@@ -630,6 +616,12 @@ vector< pair<OrientedReadId, shasta::AlignmentInfo> >
         OrientedReadId orientedReadId1(ad.readIds[1], ad.isSameStrand ? 0 : 1);
         AlignmentInfo alignmentInfo = ad.info;
 
+        // Skip it if it is not in the read graph and only alignments
+        // in the read graph were requested.
+        if(inReadGraphOnly and (not alignmentInfo.isInReadGraph)) {
+            continue;
+        }
+
         // Swap oriented reads, if necessary.
         if(orientedReadId0.getReadId() != readId0) {
             swap(orientedReadId0, orientedReadId1);
@@ -661,6 +653,7 @@ void Assembler::flagPalindromicReads(
     double alignedFractionThreshold,
     double nearDiagonalFractionThreshold,
     uint32_t deltaThreshold,
+    bool writeToCsv,
     size_t threadCount)
 {
     cout << timestamp << "Finding palindromic reads." << endl;
@@ -703,13 +696,18 @@ void Assembler::flagPalindromicReads(
         double(palindromicReadCount)/double(readCount) << endl;
 
 
-    // Write a csv file with the list of palindromic reads.
+    // Write a csv file with the list of palindromic reads
+    // if shasta is running in "filterReads" mode.
     // This should not too big as the typical rate of
     // palindromic reads is around 1e-4.
-    ofstream csvOut("PalindromicReads.csv");
-    for(ReadId readId=0; readId<readCount; readId++) {
-        if(reads->getFlags(readId).isPalindromic) {
-            csvOut << readId << "\n";
+    if(writeToCsv) {
+        // Open the file in "append" mode because it may already have
+        // been created during read loading
+        ofstream csvOut("PalindromicReads.csv", std::ofstream::app);
+        for(ReadId readId = 0; readId < readCount; readId++) {
+            if(reads->getFlags(readId).isPalindromic) {
+                csvOut << reads->getReadName(readId) << "\n";
+            }
         }
     }
 }
@@ -1028,6 +1026,70 @@ uint32_t Assembler::countCommonMarkersWithOffsetIn(
 }
 
 
+void Assembler::writeAlignmentDetails() const
+{
+    string directoryName = "Alignments/";
+    string header = "kmerId,ordinal0,ordinal1,rlePosition0,rlePosition1,";
+
+    filesystem::createDirectory(directoryName);
+
+    for (uint32_t alignmentIndex=0; alignmentIndex<alignmentData.size(); alignmentIndex++){
+        // Access the stored information we have about this alignment.
+        AlignmentData alignmentDatum = alignmentData[alignmentIndex];
+        span<const char> compressedAlignment = compressedAlignments[alignmentIndex];
+
+        Alignment alignment;
+        decompress(compressedAlignment, alignment);
+        OrientedReadId orientedReadId0 = OrientedReadId(alignmentDatum.readIds[0], 0);
+        OrientedReadId orientedReadId1 = OrientedReadId(alignmentDatum.readIds[1], alignmentDatum.isSameStrand ? 0 : 1);
+
+        string name0 = string(reads->getReadName(orientedReadId0.getReadId()).begin(),
+                              reads->getReadName(orientedReadId0.getReadId()).end());
+
+        string name1 = string(reads->getReadName(orientedReadId1.getReadId()).begin(),
+                              reads->getReadName(orientedReadId1.getReadId()).end());
+
+        string filename = name0 + "_" + name1 + "_" + (alignmentDatum.isSameStrand ? "1" : "0") + ".csv";
+
+        // Create a writeable a csv file
+        ofstream csv(directoryName + filename);
+        if (not (csv.is_open() and csv.good())){
+            throw runtime_error("ERROR: file could not be written: " + directoryName + filename);
+        }
+
+        csv << header << '\n';
+
+        // Access the markers for the two oriented reads.
+        const auto markers0 = markers[orientedReadId0.getValue()];
+        const auto markers1 = markers[orientedReadId1.getValue()];
+
+        // Compute the raw position corresponding to each RLE position.
+        const vector<uint32_t> rawPositions0 = reads->getRawPositions(orientedReadId0);
+        const vector<uint32_t> rawPositions1 = reads->getRawPositions(orientedReadId1);
+
+        // Loop over all markers.
+        for(const auto& ordinals: alignment.ordinals) {
+            const auto ordinal0 = ordinals[0];
+            const auto ordinal1 = ordinals[1];
+
+            const auto& marker0 = markers0[ordinal0];
+            const auto& marker1 = markers1[ordinal1];
+
+            const uint32_t rlePosition0 = marker0.position;
+            const uint32_t rlePosition1 = marker1.position;
+
+            const auto kmerId = marker0.kmerId;
+            SHASTA_ASSERT(marker1.kmerId == kmerId);
+
+            csv << kmerId << ','
+                << ordinal0 << ','
+                << ordinal1 << ','
+                << rlePosition0 << ','
+                << rlePosition1 << ',' << '\n';
+        }
+    }
+}
+
 
 // Check if an alignment between two reads should be suppressed,
 // bases on the setting of command line option
