@@ -1,9 +1,11 @@
 // Shasta.
 #include "Assembler.hpp"
+#include "assembleMarkerGraphPath.hpp"
 #include "AssembledSegment.hpp"
 #include "deduplicate.hpp"
 #include "LocalAssemblyGraph.hpp"
 #include "orderPairs.hpp"
+#include "Reads.hpp"
 #include "timestamp.hpp"
 using namespace shasta;
 
@@ -241,8 +243,8 @@ void Assembler::createAssemblyGraphEdges()
     assemblyGraph.createMarkerToAssemblyTable(edges.size());
 
 
-
-    // Create a histogram of size (chain length) of assembly graph vertices.
+#if 0
+    // Create a histogram of size (chain length) of assembly graph edges.
     vector<size_t> histogram;
     for(EdgeId edgeId=0; edgeId<assemblyGraph.edgeLists.size(); edgeId++) {
         const size_t size = assemblyGraph.edgeLists.size(edgeId);
@@ -259,6 +261,7 @@ void Assembler::createAssemblyGraphEdges()
             csv << size << "," << frequency << "\n";
         }
     }
+#endif
 }
 
 
@@ -467,6 +470,12 @@ void Assembler::removeLowCoverageCrossEdges(uint32_t crossEdgeCoverageThreshold)
         }
     }
 
+    const bool debug = false;
+    ofstream out;
+    if(debug) {
+        out.open("LowCoverageCrossEdges.csv");
+    }
+
     // Process assembly graph edges in order of increasing coverage.
     uint64_t removedAssemblyGraphEdgeCount = 0;
     uint64_t removedMarkerGraphEdgeCount = 0;
@@ -477,22 +486,28 @@ void Assembler::removeLowCoverageCrossEdges(uint32_t crossEdgeCoverageThreshold)
             const AssemblyGraph::VertexId v1 = edge.target;
 
             // If it is not a cross-edge, skip.
-            const size_t inDegree0 = assemblyGraph.edgesByTarget.size(v0);
+            /*
+            // Ok if more than one edge coming in.
+            const size_t inDegree0 = assemblyGraph.inDegree(v0);
             if(!(inDegree0 == 1)) {
                 continue;
             }
-            const size_t outDegree0 = assemblyGraph.edgesBySource.size(v0);
+            */
+            const size_t outDegree0 = assemblyGraph.outDegree(v0);
             if(!(outDegree0 > 1)) {
                 continue;
             }
-            const size_t inDegree1 = assemblyGraph.edgesByTarget.size(v1);
+            const size_t inDegree1 = assemblyGraph.inDegree(v1);
             if(!(inDegree1 > 1)) {
                 continue;
             }
-            const size_t outDegree1 = assemblyGraph.edgesBySource.size(v1);
+            /*
+            // Ok if more than one edge going out.
+            const size_t outDegree1 = assemblyGraph.outDegree(v1);
             if(!(outDegree1 == 1)) {
                 continue;
             }
+            */
 
             // Mark this edge.
             edge.removalReason = AssemblyGraph::Edge::RemovalReason::LowCoverageCrossEdge;
@@ -501,6 +516,9 @@ void Assembler::removeLowCoverageCrossEdges(uint32_t crossEdgeCoverageThreshold)
             // Mark the corresponding marker graph edges.
             for(const MarkerGraph::EdgeId markerGraphEdgeId: assemblyGraph.edgeLists[edgeId]) {
                 markerGraph.edges[markerGraphEdgeId].isLowCoverageCrossEdge = 1;
+                if(debug) {
+                    out << markerGraphEdgeId << "\n";
+                }
                 ++removedMarkerGraphEdgeCount;
             }
 
@@ -727,7 +745,7 @@ void Assembler::assemble(
     }
     cout << timestamp << "Assembled a total " << totalBaseCount <<
         " bases for " << assemblyGraph.edgeLists.size() << " assembly graph edges of which " <<
-        assembledEdgeCount << " where assembled." << endl;
+        assembledEdgeCount << " were assembled." << endl;
 
 }
 
@@ -845,31 +863,6 @@ void Assembler::accessAssemblyGraphSequences()
 
 
 
-void Assembler::findAssemblyGraphBubbles()
-{
-    AssemblyGraph& assemblyGraph = *assemblyGraphPointer;
-
-    // Check that we have what we need.
-    SHASTA_ASSERT(assemblyGraph.vertices.isOpen);
-    SHASTA_ASSERT(assemblyGraph.edges.isOpen);
-    SHASTA_ASSERT(assemblyGraph.edgesBySource.isOpen());
-    SHASTA_ASSERT(assemblyGraph.edgesByTarget.isOpen());
-
-    // Find the bubbles.
-    assemblyGraph.bubbles.createNew(
-        largeDataName("AssemblyGraphBubbles"),
-        largeDataPageSize);
-    assemblyGraph.findBubbles();
-
-    // Find the bubble chains.
-    assemblyGraph.bubbleChains.createNew(
-        largeDataName("AssemblyGraphBubbleChains"),
-        largeDataPageSize);
-    assemblyGraph.findBubbleChains();
-}
-
-
-
 void Assembler::computeAssemblyStatistics()
 {
     AssemblyGraph& assemblyGraph = *assemblyGraphPointer;
@@ -1261,6 +1254,14 @@ void Assembler::writeGfa1BothStrands(const string& fileName)
 }
 
 
+
+void Assembler::writeGfa1BothStrandsNoSequence(const string& fileName)
+{
+    assemblyGraphPointer->writeGfa1BothStrandsNoSequence(fileName);
+}
+
+
+
 // Write assembled sequences in FASTA format.
 void Assembler::writeFasta(const string& fileName)
 {
@@ -1774,148 +1775,20 @@ void Assembler::assembleAssemblyGraphEdge(
     AssembledSegment& assembledSegment)
 {
     AssemblyGraph& assemblyGraph = *assemblyGraphPointer;
-
-    assembledSegment.clear();
-    const auto k = assemblerInfo->k;
-    assembledSegment.k = k;
-    assembledSegment.assemblyGraphEdgeId = edgeId;
-
-    // The edges of this chain in the marker graph.
-    const span<MarkerGraph::EdgeId> assemblerEdgeIds = assemblyGraph.edgeLists[edgeId];
-    assembledSegment.edgeCount = assemblerEdgeIds.size();
-    assembledSegment.vertexCount = assembledSegment.edgeCount + 1;
-    assembledSegment.edgeIds.resize(assembledSegment.edgeCount);
-    copy(assemblerEdgeIds.begin(), assemblerEdgeIds.end(), assembledSegment.edgeIds.begin());
-
-    // Gather the vertices of this chain in the marker graph.
-    assembledSegment.vertexIds.reserve(assembledSegment.vertexCount);
-    for(const MarkerGraph::EdgeId edgeId: assembledSegment.edgeIds) {
-        const MarkerGraph::Edge& edge =
-            markerGraph.edges[edgeId];
-        assembledSegment.vertexIds.push_back(edge.source);
-    }
-    const MarkerGraph::Edge& lastEdge =
-        markerGraph.edges[assembledSegment.edgeIds[assembledSegment.edgeIds.size()-1]];
-    assembledSegment.vertexIds.push_back(lastEdge.target);
-
-    // Get vertex coverage.
-    assembledSegment.vertexCoverage.resize(assembledSegment.vertexCount);
-    for(size_t i=0; i<assembledSegment.vertexCount; i++) {
-        assembledSegment.vertexCoverage[i] = uint32_t(markerGraph.vertexCoverage(assembledSegment.vertexIds[i]));
-    }
-
-    // Edge coverage.
-    assembledSegment.edgeCoverage.resize(assembledSegment.edgeCount);
-    for(size_t i=0; i<assembledSegment.edgeCount; i++) {
-        assembledSegment.edgeCoverage[i] =
-            uint32_t(markerGraph.edgeMarkerIntervals.size(assembledSegment.edgeIds[i]));
-    }
+    const span<const MarkerGraph::EdgeId> markerGraphPath = makeSpanOfConst(assemblyGraph.edgeLists[edgeId]);
+    assembleAssemblyGraphEdge(markerGraphPath, storeCoverageData, assembledSegment);
+}
 
 
 
-    // Extract consensus sequence for the vertices of the chain.
-    assembledSegment.vertexSequences.resize(assembledSegment.vertexCount);
-    assembledSegment.vertexRepeatCounts.resize(assembledSegment.vertexCount);
-    for(size_t i=0; i<assembledSegment.vertexCount; i++) {
-
-        // Get the sequence.
-        const MarkerId firstMarkerId = markerGraph.getVertexMarkerIds(assembledSegment.vertexIds[i])[0];
-        const CompressedMarker& firstMarker = markers.begin()[firstMarkerId];
-        const KmerId kmerId = firstMarker.kmerId;
-        const Kmer kmer(kmerId, assemblerInfo->k);
-
-        // Get the repeat counts.
-        const auto& storedConsensus = markerGraph.vertexRepeatCounts.begin() + k * assembledSegment.vertexIds[i];
-
-        // Store in the AssembledSegment.
-        assembledSegment.vertexSequences[i].resize(k);
-        assembledSegment.vertexRepeatCounts[i].resize(k);
-        for(size_t j=0; j<k; j++) {
-            assembledSegment.vertexSequences[i][j] = kmer[j];
-            assembledSegment.vertexRepeatCounts[i][j] = storedConsensus[j];
-        }
-    }
-
-
-
-    // Extract consensus sequence for the edges of the chain.
-    assembledSegment.edgeSequences.resize(assembledSegment.edgeCount);
-    assembledSegment.edgeRepeatCounts.resize(assembledSegment.edgeCount);
-    assembledSegment.edgeOverlappingBaseCounts.resize(assembledSegment.edgeCount);
-    for(size_t i=0; i<assembledSegment.edgeCount; i++) {
-
-        const auto& storedConsensus = markerGraph.edgeConsensus[assembledSegment.edgeIds[i]];
-        assembledSegment.edgeSequences[i].resize(storedConsensus.size());
-        assembledSegment.edgeRepeatCounts[i].resize(storedConsensus.size());
-        for(size_t j=0; j<storedConsensus.size(); j++) {
-            assembledSegment.edgeSequences[i][j] = storedConsensus[j].first;
-            assembledSegment.edgeRepeatCounts[i][j] = storedConsensus[j].second;
-        }
-        assembledSegment.edgeOverlappingBaseCounts[i] =
-            markerGraph.edgeConsensusOverlappingBaseCount[assembledSegment.edgeIds[i]];
-    }
-
-
-
-    // Extract coverage data for vertices and edges.
-    if(storeCoverageData) {
-
-        // Check that coverage data is available.
-        if( !markerGraph.vertexCoverageData.isOpen() ||
-            !markerGraph.edgeCoverageData.isOpen()) {
-            throw runtime_error("Coverage data is not accessible.");
-        }
-
-        // Vertices.
-        assembledSegment.vertexCoverageData.resize(assembledSegment.vertexCount);
-        for(size_t i=0; i<assembledSegment.vertexCount; i++) {
-            const auto& input = markerGraph.vertexCoverageData[assembledSegment.vertexIds[i]];
-            auto& output = assembledSegment.vertexCoverageData[i];
-            output.resize(k);
-            for(const pair<uint32_t, CompressedCoverageData>& p: input) {
-                const uint32_t position = p.first;
-                SHASTA_ASSERT(position < k);
-                const CompressedCoverageData& cd = p.second;
-                output[position].push_back(cd);
-            }
-        }
-
-        // Edges.
-        assembledSegment.edgeCoverageData.resize(assembledSegment.edgeCount);
-        for(size_t i=0; i<assembledSegment.edgeCount; i++) {
-            const auto& input = markerGraph.edgeCoverageData[assembledSegment.edgeIds[i]];
-            auto& output = assembledSegment.edgeCoverageData[i];
-            for(const pair<uint32_t, CompressedCoverageData>& p: input) {
-                const uint32_t position = p.first;
-                if(position >= output.size()) {
-                    output.resize(position+1);
-                }
-                const CompressedCoverageData& cd = p.second;
-                output[position].push_back(cd);
-            }
-        }
-    }
-
-
-
-    // Compute vertex offsets.
-    // A vertex offset is the position of the first base
-    // of the vertex consensus sequence (run-length)
-    // relative to the first base of assembled sequence (run-length).
-    assembledSegment.computeVertexOffsets();
-
-    // Compute, for each vertex, the portion of vertex sequence that contributes
-    // to the assembly. This is the portion that does not overlap a vertex with greater coverage.
-    // (Break ties using vertex ids).
-    // An edge with overlapping markers does not contribute to the assembly.
-    // An edge with at least one intervening base contributes all of its bases
-    // to the assembly.
-    assembledSegment.computeVertexAssembledPortion();
-
-    // Assemble run-length sequence and raw sequence.
-    // Keep track of the range each vertex and edge contributes.
-    assembledSegment.assemble();
-
+void Assembler::assembleAssemblyGraphEdge(
+    const span<const MarkerGraph::EdgeId>& markerGraphPath,
+    bool storeCoverageData,
+    AssembledSegment& assembledSegment)
+{
+    assembleMarkerGraphPath(
+        assemblerInfo->k, markers, markerGraph, markerGraphPath,
+        storeCoverageData, assembledSegment);
 }
 
 
