@@ -1,8 +1,12 @@
 
 // Shasta.
 #include "Assembler.hpp"
+#include "AssemblyGraph.hpp"
+#include "Coverage.hpp"
 #include "buildId.hpp"
+#include "filesystem.hpp"
 #include "platformDependent.hpp"
+#include "Reads.hpp"
 using namespace shasta;
 
 // Boost libraries.
@@ -10,6 +14,180 @@ using namespace shasta;
 #include <boost/uuid/uuid_generators.hpp>
 #include <boost/uuid/uuid_io.hpp>
 
+// Standard library.
+#include <filesystem>
+
+
+// A map containing descriptions of output files.
+namespace shasta {
+    std::map<string, string> outputDescriptionTable = {
+        {
+            "index.html",
+            "Html file containing a list of assembly output files and their descriptions."
+        },
+        {
+            "AssemblySummary.json",
+            "Assembly summary information in json format."
+        },
+        {
+            "AssemblySummary.html",
+            "Assembly summary information in html format."
+        },
+        {
+            "Assembly.fasta",
+            "Assembly in FASTA format (one strand only)."
+        },
+        {
+            "Assembly.gfa",
+            "Assembly in GFA format (one strand only)."
+        },
+        {
+            "Assembly-BothStrands.gfa",
+            "Assembly in GFA format (both strands)."
+        },
+        {
+            "Assembly-BothStrands-NoSequence.gfa",
+            "Assembly in GFA format (compact output without sequence, both strands)."
+        },
+        {
+            "AssemblySummary.csv",
+            "List of assembled segments in order of decreasing length."
+        },
+        {
+            "Binned-ReadLengthHistogram.csv",
+            "Read length distribution in 1 kb bins."
+        },
+        {
+            "ReadLengthHistogram.csv",
+            "Detailed read length distribution."
+        },
+        {
+            "ReadSummary.csv",
+            "Summary file containing one line of information for each read."
+        },
+        {
+            "LowHashBucketHistogram.csv",
+            "MinHash bucket population histogram."
+        },
+        {
+            "DisjointSetsHistogram.csv",
+            "Coverage histogram for all disjoint sets."
+        },
+        {
+            "MarkerGraphVertexCoverageHistogram.csv",
+            "Coverage histogram for marker graph vertices."
+        },
+        {
+            "MarkerGraphEdgeCoverageHistogram.csv",
+            "Coverage histogram for marker graph edges."
+        },
+        {
+            "SuppressedAlignmentCandidates.csv",
+            "Details of suppressed alignment candidates."
+        },
+        {
+            "ReadGraphComponents.csv",
+            "Information about connected components of the read graph."
+        },
+        {
+            "shasta.conf",
+            "Configuration file containing options in effect for this assembly."
+        },
+        {
+            "ReadLowHashStatistics.csv",
+            "MinHash/LowHash statistics for each read."
+        },
+        {
+            "AlignedFractionHistogram.csv",
+            "Histogram of aligned fraction for all computed alignments."
+        },
+        {
+            "AlignmentDriftHistogram.csv",
+            "Histogram of marker drift for all computed alignments."
+        },
+        {
+            "AlignmentMarkerCountHistogram.csv",
+            "Histogram of number of aligned markers for all computed alignments."
+        },
+        {
+            "AlignmentSkipHistogram.csv",
+            "Histogram of marker skip for all computed alignments."
+        },
+        {
+            "AlignmentTrimHistogram.csv",
+            "Histogram of marker trim for all computed alignments."
+        },
+        {
+            "BubbleChains.csv",
+            "Information about bubble chains in the assembly."
+        },
+        {
+            "PhasingRegions.csv",
+            "Information about Phasing regions in the assembly."
+        },
+        {
+            "Assembly-Detailed-NoSequence.gfa",
+            "Detailed assembly representation with small bubbles: "
+            "compact GFA file without sequence."
+        },
+        {
+            "Assembly-Detailed.csv",
+            "Detailed assembly representation with small bubbles: "
+            "csv companion for the GFA files."
+        },
+        {
+            "Assembly-Detailed.fasta",
+            "Detailed assembly representation with small bubbles: "
+            "FASTA file."
+        },
+        {
+            "Assembly-Detailed.gfa",
+            "Detailed assembly representation with small bubbles: "
+            "complete GFA file."
+        },
+        {
+            "Assembly-Haploid-NoSequence.gfa",
+            "Haploid assembly representation: "
+            "compact GFA file without sequence."
+        },
+        {
+            "Assembly-Haploid.csv",
+            "Haploid assembly representation: "
+            "csv companion for the GFA files."
+        },
+        {
+            "Assembly-Haploid.fasta",
+            "Haploid assembly representation: "
+            "FASTA file."
+        },
+        {
+            "Assembly-Haploid.gfa",
+            "Haploid assembly representation: "
+            "complete GFA file."
+        },
+        {
+            "Assembly-Phased-NoSequence.gfa",
+            "Phased assembly representation with large bubbles: "
+            "compact GFA file without sequence."
+        },
+        {
+            "Assembly-Phased.csv",
+            "Phased assembly representation with large bubbles: "
+            "csv companion for the GFA files."
+        },
+        {
+            "Assembly-Phased.fasta",
+            "Phased assembly representation with large bubbles: "
+            "FASTA file."
+        },
+        {
+            "Assembly-Phased.gfa",
+            "Phased assembly representation with large bubbles: "
+            "complete GFA file."
+        }
+    };
+}
+
 
 
 #ifdef SHASTA_HTTP_SERVER
@@ -29,8 +207,10 @@ void Assembler::fillServerFunctionTable()
     SHASTA_ADD_TO_FUNCTION_TABLE(exploreRead);
     SHASTA_ADD_TO_FUNCTION_TABLE(blastRead);
     SHASTA_ADD_TO_FUNCTION_TABLE(exploreAlignments);
+    SHASTA_ADD_TO_FUNCTION_TABLE(exploreAlignmentCoverage);
     SHASTA_ADD_TO_FUNCTION_TABLE(exploreAlignment);
     SHASTA_ADD_TO_FUNCTION_TABLE(computeAllAlignments);
+    SHASTA_ADD_TO_FUNCTION_TABLE(exploreAlignmentCandidateGraph);
     SHASTA_ADD_TO_FUNCTION_TABLE(exploreAlignmentGraph);
     SHASTA_ADD_TO_FUNCTION_TABLE(alignSequencesInBaseRepresentation);
     SHASTA_ADD_TO_FUNCTION_TABLE(alignSequencesInMarkerRepresentation);
@@ -41,6 +221,8 @@ void Assembler::fillServerFunctionTable()
     SHASTA_ADD_TO_FUNCTION_TABLE(exploreMarkerGraphEdge);
     SHASTA_ADD_TO_FUNCTION_TABLE(exploreMarkerCoverage);
     SHASTA_ADD_TO_FUNCTION_TABLE(exploreMarkerGraphInducedAlignment);
+    SHASTA_ADD_TO_FUNCTION_TABLE(followReadInMarkerGraph);
+    SHASTA_ADD_TO_FUNCTION_TABLE(exploreMarkerConnectivity);
     SHASTA_ADD_TO_FUNCTION_TABLE(exploreAssemblyGraph);
     SHASTA_ADD_TO_FUNCTION_TABLE(exploreAssemblyGraphEdge);
     SHASTA_ADD_TO_FUNCTION_TABLE(exploreAssemblyGraphEdgesSupport);
@@ -126,8 +308,15 @@ void Assembler::writeMakeAllTablesCopyable(ostream& html) const
     // Copy to the clipboard the table that generated the event.
     function copyToClipboard(event)
     {
-        // Turn off the standard context menu.
-        event.preventDefault();
+        // If the CTRL key is not pressed, don't do anything.
+        if(!event.ctrlKey) {
+            return;
+        }
+
+        // Prevent default behavior.
+        // event.preventDefault();
+        // event.stopPropagation();
+        // event.returnValue = false;
         
         // Get the table element.
         var element = event.currentTarget;
@@ -143,16 +332,21 @@ void Assembler::writeMakeAllTablesCopyable(ostream& html) const
         
         // Copy it to the clipboard.
         document.execCommand("copy");
+
+        // Unselect it.
+        selection.removeAllRanges();
+
+        window.alert("The table was copied to the clipboard");
     }
 
-    // Make a table copyable by right click.
+    // Make a table copyable by Ctrl-click.
     function makeCopyable(element)
     {
-        element.addEventListener('contextmenu', copyToClipboard);
-        element.title = 'Right click anywhere on the table to copy the entire table to the clipboard';
+        element.addEventListener('click', copyToClipboard);
+        element.title = 'Ctrl-click anywhere on the table to copy the entire table to the clipboard';
     }
 
-    // Make all tables copyable by right click.
+    // Make all tables copyable by Ctrl-click.
     function makeAllTablesCopyable()
     {
         var tables = document.getElementsByTagName('table');
@@ -218,7 +412,9 @@ void Assembler::writeNavigation(ostream& html) const
         {"Reads", "exploreRead"},
         });
     writeNavigation(html, "Alignments", {
+        {"Candidate graph", "exploreAlignmentCandidateGraph"},
         {"Stored alignments", "exploreAlignments"},
+        {"Alignment coverage", "exploreAlignmentCoverage"},
         {"Align two reads", "exploreAlignment"},
         {"Align one read with all", "computeAllAlignments"},
         {"Alignment graph", "exploreAlignmentGraph"},
@@ -235,6 +431,8 @@ void Assembler::writeNavigation(ostream& html) const
         {"Marker graph edges", "exploreMarkerGraphEdge"},
         {"Marker coverage", "exploreMarkerCoverage"},
         {"Induced alignments", "exploreMarkerGraphInducedAlignment"},
+        {"Follow a read in the marker graph", "followReadInMarkerGraph"},
+        {"Marker connectivity", "exploreMarkerConnectivity"},
         });
     writeNavigation(html, "Assembly graph", {
         {"Local assembly graph", "exploreAssemblyGraph"},
@@ -380,6 +578,13 @@ void Assembler::accessAllSoft()
         allDataAreAvailable = false;
     }
 
+    try {
+        accessAlignmentCandidateTable();
+    } catch(const exception& e) {
+        cout << "Alignment candidate table is not accessible." << endl;
+        allDataAreAvailable = false;
+    }
+
     try {
         accessReadLowHashStatistics();
     } catch(const exception& e) {
@@ -414,6 +619,13 @@ void Assembler::accessAllSoft()
         allDataAreAvailable = false;
     }
 
+    try {
+        accessMarkerGraphReverseComplementVertex();
+    } catch(const exception& e) {
+        cout << "Marker graph reverse complement vertices are not accessible." << endl;
+        allDataAreAvailable = false;
+    }
+
     try {
         accessMarkerGraphEdges(false);
     } catch(const exception& e) {
@@ -421,10 +633,17 @@ void Assembler::accessAllSoft()
         allDataAreAvailable = false;
     }
 
+    try {
+        accessMarkerGraphReverseComplementEdge();
+    } catch(const exception& e) {
+        cout << "Marker graph reverse complement edges are not accessible." << endl;
+        allDataAreAvailable = false;
+    }
+
     try {
         accessMarkerGraphConsensus();
     } catch(const exception& e) {
-        cout << "MarkerGraph graph consensus is accessible." << endl;
+        cout << "MarkerGraph graph consensus is not accessible." << endl;
         allDataAreAvailable = false;
     }
 
@@ -618,14 +837,14 @@ void Assembler::writeHtmlEnd(ostream& html) const
 
 
 
-void Assembler::writeAssemblySummary(ostream& html)
+void Assembler::writeAssemblySummary(ostream& html, bool readsOnly)
 {
     writeHtmlBegin(html);
-    writeAssemblySummaryBody(html);
+    writeAssemblySummaryBody(html, readsOnly);
     writeHtmlEnd(html);
 }
 
-void Assembler::writeAssemblySummaryBody(ostream& html)
+void Assembler::writeAssemblySummaryBody(ostream& html, bool readsOnly)
 {
     using std::setprecision;
     AssemblyGraph& assemblyGraph = *assemblyGraphPointer;
@@ -646,11 +865,13 @@ void Assembler::writeAssemblySummaryBody(ostream& html)
     const uint64_t totalDiscardedReadCount =
         assemblerInfo->discardedInvalidBaseReadCount +
         assemblerInfo->discardedShortReadReadCount +
-        assemblerInfo->discardedBadRepeatCountReadCount;
+        assemblerInfo->discardedBadRepeatCountReadCount +
+        assemblerInfo->discardedPalindromicReadCount;
     const uint64_t totalDiscardedBaseCount =
         assemblerInfo->discardedInvalidBaseBaseCount +
         assemblerInfo->discardedShortReadBaseCount +
-        assemblerInfo->discardedBadRepeatCountBaseCount;
+        assemblerInfo->discardedBadRepeatCountBaseCount +
+        assemblerInfo->discardedPalindromicBaseCount;
 
 
     html <<
@@ -676,7 +897,7 @@ void Assembler::writeAssemblySummaryBody(ostream& html)
         "<td class=right>" << reads->getRepeatCountsTotalSize() <<
         "<tr><td>Average length ratio of run-length encoded sequence over raw sequence"
         "<td class=right>" << setprecision(4) << double(reads->getRepeatCountsTotalSize()) / double(assemblerInfo->baseCount) <<
-        "<tr><td>Number of reads flagged as palindromic"
+        "<tr><td>Number of reads flagged as palindromic by self alignment"
         "<td class=right>" << assemblerInfo->palindromicReadCount <<
         "<tr><td>Number of reads flagged as chimeric"
         "<td class=right>" << assemblerInfo->chimericReadCount <<
@@ -702,6 +923,9 @@ void Assembler::writeAssemblySummaryBody(ostream& html)
         "<tr><td>Reads discarded on input because they contained repeat counts greater than 255"
         "<td class=right>" << assemblerInfo->discardedBadRepeatCountReadCount <<
         "<td class=right>" << assemblerInfo->discardedBadRepeatCountBaseCount <<
+        "<tr><td>Reads discarded on input because they had quality scores indicative of palindromic sequence"
+        "<td class=right>" << assemblerInfo->discardedPalindromicReadCount <<
+        "<td class=right>" << assemblerInfo->discardedPalindromicBaseCount <<
         "<tr><td>Reads discarded on input, total"
         "<td class=right>" <<totalDiscardedReadCount <<
         "<td class=right>" <<totalDiscardedBaseCount <<
@@ -717,9 +941,13 @@ void Assembler::writeAssemblySummaryBody(ostream& html)
         "</table>"
         "<ul><li>Base counts in the above table are raw sequence bases."
         "<li>Here and elsewhere, &quot;raw&quot; refers to the original read sequence, "
-        "as opposed to run-length encoded sequence.</ul>"
+        "as opposed to run-length encoded sequence.</ul>";
 
+    if (readsOnly) {
+        return;
+    }
 
+    html <<
         "<h3>Marker <i>k</i>-mers</h3>"
         "<table>"
         "<tr><td>Length <i>k</i> of <i>k</i>-mers used as markers"
@@ -813,23 +1041,26 @@ void Assembler::writeAssemblySummaryBody(ostream& html)
         "<tr><td>Number of edges that were not removed"
         "<td class=right>" << assemblerInfo->markerGraphEdgesNotRemovedCount <<
         "</table>"
-        "<ul><li>The marker graph contains both strands.</ul>"
+        "<ul><li>The marker graph contains both strands.</ul>";
 
 
-
-        "<h3>Assembly graph</h3>"
-        "<table>"
-        "<tr><td>Number of vertices"
-        "<td class=right>" << assemblyGraph.vertices.size() <<
-        "<tr><td>Number of edges"
-        "<td class=right>" << assemblyGraph.edges.size() <<
-        "<tr><td>Number of edges assembled"
-        "<td class=right>" << assemblerInfo->assemblyGraphAssembledEdgeCount <<
-        "</table>"
-        "<ul><li>The assembly graph contains both strands.</ul>"
+        if(assemblyGraphPointer) {
+            html <<
+                "<h3>Assembly graph</h3>"
+                "<table>"
+                "<tr><td>Number of vertices"
+                "<td class=right>" << assemblyGraph.vertices.size() <<
+                "<tr><td>Number of edges"
+                "<td class=right>" << assemblyGraph.edges.size() <<
+                "<tr><td>Number of edges assembled"
+                "<td class=right>" << assemblerInfo->assemblyGraphAssembledEdgeCount <<
+                "</table>"
+                "<ul><li>The assembly graph contains both strands.</ul>";
+        }
 
 
 
+        html <<
         "<h3>Assembled segments (&quot;contigs&quot;)</h3>"
         "<table>"
         "<tr><td>Number of segments assembled"
@@ -869,7 +1100,7 @@ void Assembler::writeAssemblySummaryBody(ostream& html)
 
 
 
-void Assembler::writeAssemblySummaryJson(ostream& json)
+void Assembler::writeAssemblySummaryJson(ostream& json, bool readsOnly)
 {
     AssemblyGraph& assemblyGraph = *assemblyGraphPointer;
     using std::setprecision;
@@ -891,10 +1122,12 @@ void Assembler::writeAssemblySummaryJson(ostream& json)
     const uint64_t totalDiscardedReadCount =
         assemblerInfo->discardedInvalidBaseReadCount +
         assemblerInfo->discardedShortReadReadCount +
+        assemblerInfo->discardedPalindromicReadCount +
         assemblerInfo->discardedBadRepeatCountReadCount;
     const uint64_t totalDiscardedBaseCount =
         assemblerInfo->discardedInvalidBaseBaseCount +
         assemblerInfo->discardedShortReadBaseCount +
+        assemblerInfo->discardedPalindromicBaseCount +
         assemblerInfo->discardedBadRepeatCountBaseCount;
 
 
@@ -919,7 +1152,7 @@ void Assembler::writeAssemblySummaryJson(ostream& json)
         "    \"Number of run-length encoded bases\": " << reads->getRepeatCountsTotalSize() << ",\n"
         "    \"Average length ratio of run-length encoded sequence over raw sequence\": " <<
         setprecision(4) << double(reads->getRepeatCountsTotalSize()) / double(assemblerInfo->baseCount) << ",\n"
-        "    \"Number of reads flagged as palindromic\": " << assemblerInfo->palindromicReadCount << ",\n"
+        "    \"Number of reads flagged as palindromic by self alignment\": " << assemblerInfo->palindromicReadCount << ",\n"
         "    \"Number of reads flagged as chimeric\": " << assemblerInfo->chimericReadCount << "\n"
         "  },\n"
 
@@ -942,6 +1175,11 @@ void Assembler::writeAssemblySummaryJson(ostream& json)
         "      \"Reads\": " << assemblerInfo->discardedBadRepeatCountReadCount << ",\n"
         "      \"Bases\": " << assemblerInfo->discardedBadRepeatCountBaseCount << "\n"
         "    },\n"
+        "    \"Reads discarded on input because they they had quality scores indicative of palindromic sequence\":\n"
+        "    {\n"
+        "      \"Reads\": " << assemblerInfo->discardedPalindromicReadCount << ",\n"
+        "      \"Bases\": " << assemblerInfo->discardedPalindromicBaseCount << "\n"
+        "    },\n"
         "    \"Reads discarded on input, total\":\n"
         "    {\n"
         "      \"Reads\": " << totalDiscardedReadCount << ",\n"
@@ -958,10 +1196,13 @@ void Assembler::writeAssemblySummaryJson(ostream& json)
         double(totalDiscardedBaseCount + assemblerInfo->baseCount)
         << "\n"
         "    }\n"
-        "  },\n"
-
+        "  },\n";
 
+    if (readsOnly) {
+        return;
+    }
 
+    json <<
         "  \"Marker k-mers\":\n"
         "  {\n"
         "    \"Length k of k-mers used as markers\": " << assemblerInfo->k << ",\n"
@@ -1038,19 +1279,21 @@ void Assembler::writeAssemblySummaryJson(ostream& json)
         "    \"Number of vertices that are not isolated after edge removal\": " <<
         assemblerInfo->markerGraphVerticesNotIsolatedCount << ",\n"
         "    \"Number of edges that were not removed\": " << assemblerInfo->markerGraphEdgesNotRemovedCount << "\n"
-        "  },\n"
+        "  },\n";
 
 
-
-        "  \"Assembly graph\":\n"
-        "  {\n"
-        "    \"Number of vertices\": " << assemblyGraph.vertices.size() << ",\n"
-        "    \"Number of edges\": " << assemblyGraph.edges.size() << ",\n"
-        "    \"Number of edges assembled\": " << assemblerInfo->assemblyGraphAssembledEdgeCount << "\n"
-        "  },\n"
-
+    if(assemblyGraphPointer) {
+        json <<
+            "  \"Assembly graph\":\n"
+            "  {\n"
+            "    \"Number of vertices\": " << assemblyGraph.vertices.size() << ",\n"
+            "    \"Number of edges\": " << assemblyGraph.edges.size() << ",\n"
+            "    \"Number of edges assembled\": " << assemblerInfo->assemblyGraphAssembledEdgeCount << "\n"
+            "  },\n";
+    }
 
 
+    json <<
         "  \"Assembled segments\":\n"
         "  {\n"
         "    \"Number of segments assembled\": " << assemblerInfo->assemblyGraphAssembledEdgeCount << ",\n"
@@ -1082,73 +1325,37 @@ void Assembler::writeAssemblySummaryJson(ostream& json)
 void Assembler::writeAssemblyIndex(ostream& html) const
 {
     writeHtmlBegin(html);
+    html << "<body><h1>Shasta assembly output files</h1><table>";
 
-    const string s = R"ABCDE(
-    <body>
-    <h1>Assembly output files</h1>
-    <table>
+    // Loop over files in the assembly directory,
+    // in case-insensitive alphabetic order.
+    vector<string> assemblyFiles = shasta::filesystem::directoryContents(".");
+    sort(assemblyFiles.begin(), assemblyFiles.end(),
+        [](const string& s1, const string& s2) {
+        return lexicographical_compare(
+        s1.begin(), s1.end(),
+        s2.begin(), s2.end(),
+        [](const char& c1, const char& c2) {
+        return tolower(c1) < tolower(c2);
+        });});
+    for(string file: assemblyFiles) {
 
-    <tr>
-    <td><a href='AssemblySummary.html'>AssemblySummary.html</a>
-    <td>Assembly summary information.
+        // Take out "./"
+        file = file.substr(2);
 
-    <tr>
-    <td><a href='Assembly.fasta'>Assembly.fasta</a>
-    <td>Assembly in Fasta format (one strand only).
+        // Get the description.
+        string description;
+        auto it = outputDescriptionTable.find(file);
+        if(it != outputDescriptionTable.end()) {
+            description = it->second;
+        }
 
-    <tr>
-    <td><a href='Assembly.gfa'>Assembly.gfa</a>
-    <td>Assembly in gfa format (one strand only).
+        // Write a row in the table for this file.
+        html <<
+            "<tr><td><a href='" << file << "'>" << file <<
+            "</a><td>" << description << endl;
+    }
 
-    <tr>
-    <td><a href='Assembly-BothStrands.gfa'>Assembly-BothStrands.gfa</a>
-    <td>Assembly in gfa format (both strands).
-
-    <tr>
-    <td><a href='AssemblySummary.csv'>AssemblySummary.csv</a>
-    <td>List of assembled segments in order of decreasing length.
-
-    <tr>
-    <td><a href='Binned-ReadLengthHistogram.csv'>Binned-ReadLengthHistogram.csv</a>
-    <td>Read length distribution in 1 kb bins.
-
-    <tr>
-    <td><a href='ReadLengthHistogram.csv'>ReadLengthHistogram.csv</a>
-    <td>Detailed read length distribution.
-
-    <tr>
-    <td><a href='ReadSummary.csv'>ReadSummary.csv</a>
-    <td>Summary file containing one line of information for each read.
-
-    <tr>
-    <td><a href='LowHashBucketHistogram.csv'>LowHashBucketHistogram.csv</a>
-    <td>MinHash bucket population histogram.
-
-    <tr>
-    <td><a href='DisjointSetsHistogram.csv'>DisjointSetsHistogram.csv</a>
-    <td>Coverage histogram for all disjoint sets.
-
-    <tr>
-    <td><a href='MarkerGraphVertexCoverageHistogram.csv'>MarkerGraphVertexCoverageHistogram.csv</a>
-    <td>Coverage histogram for marker graph vertices.
-
-    <tr>
-    <td><a href='MarkerGraphEdgeCoverageHistogram.csv'>MarkerGraphEdgeCoverageHistogram.csv</a>
-    <td>Coverage histogram for marker graph edges.
-
-    <tr>
-    <td><a href='ReadLengthHistogram.csv'>ReadLengthHistogram.csv</a>
-    <td>Detailed read length distribution.
-
-    <tr>
-    <td><a href='SuppressedAlignmentCandidates.csv'>SuppressedAlignmentCandidates.csv</a>
-    <td>Details of suppressed alignment candidates.
-
-    </table>
-    </body>
-)ABCDE";
-
-    html << s;
     writeHtmlEnd(html);
 }
 
@@ -1162,7 +1369,7 @@ void Assembler::blastRead(
     ostream& html)
 {
 
-    if(!filesystem::isRegularFile(httpServerData.referenceFastaFileName)) {
+    if(!std::filesystem::is_regular_file(httpServerData.referenceFastaFileName)) {
         html << "<p>The fasta sequence " << httpServerData.referenceFastaFileName <<
             " to be used as the reference (Blast subject) does not exist.";
         return;
@@ -1254,7 +1461,7 @@ void Assembler::blastRead(
 
 
     // Copy any error output to html.
-    if(filesystem::fileSize(blastErrFileName)) {
+    if(std::filesystem::file_size(blastErrFileName)) {
         ifstream blastErrFile(blastErrFileName);
         html << "<pre style='font-size:10px'>";
         html << blastErrFile.rdbuf();
