@@ -1,5 +1,6 @@
 #include "Assembler.hpp"
 #include "deduplicate.hpp"
+#include "Reads.hpp"
 using namespace shasta;
 
 #include <random>
@@ -818,3 +819,338 @@ void Assembler::selectKmers2ThreadFunction(size_t threadId)
     overenrichedReadCount.remove();
 }
 
+
+
+
+// In this version, marker k-mers are selected randomly, but excluding
+// k-mers that appear repeated at short distances in any oriented read.
+// More precisely, for each k-mer we compute the minimum distance
+// (in RLE bases) at which any two copies of that k-mer appear in any oriented read.
+// K-mers for which this minimum distance is less than distanceThreshold
+// are not used as markers. Marker k-mers are selected randomly among the
+// remaining k-mers, until the desired marker density is achieved.
+void Assembler::selectKmers4(
+
+    // k-mer length.
+    uint64_t k,
+
+    // The desired marker density
+    double markerDensity,
+
+    // Seed for random number generator.
+    uint64_t seed,
+
+    // Exclude k-mers that appear in any read in two copies,
+    // with the two copies closer than this distance (in RLE bases).
+    uint64_t distanceThreshold,
+
+    size_t threadCount
+)
+{
+    const bool debug = false;
+    cout << timestamp << "Begin selectKmers4." << endl;
+
+    // Sanity check on the value of k, then store it.
+    if(k > Kmer::capacity) {
+        throw runtime_error("K-mer capacity exceeded.");
+    }
+    assemblerInfo->k = k;
+
+    // Sanity check.
+    if(markerDensity<0. || markerDensity>1.) {
+        throw runtime_error("Invalid marker density " +
+            to_string(markerDensity) + " requested.");
+    }
+
+    // Adjust the numbers of threads, if necessary.
+    if(threadCount == 0) {
+        threadCount = std::thread::hardware_concurrency();
+    }
+
+    // Fill in the fields of the k-mer table
+    // that depends only on k.
+    initializeKmerTable();
+
+    // Initialize the global frequency of all k-mers.
+    selectKmers4Data.globalFrequency.createNew(
+        largeDataName("tmp-SelectKmers4-GlobalFrequency"),  largeDataPageSize);
+    selectKmers4Data.globalFrequency.resize(kmerTable.size());
+    fill(
+        selectKmers4Data.globalFrequency.begin(),
+        selectKmers4Data.globalFrequency.end(), 0);
+
+
+    // Initialize the minimumDistance vector, which stores
+    // the minimum RLE distance between any two copies of each k-mer
+    // in any oriented read.
+    selectKmers4Data.minimumDistance.createNew(
+        largeDataName("tmp-selectKmers4-minimumDistance"), largeDataPageSize);
+    const uint64_t kmerCount = kmerTable.size();
+    selectKmers4Data.minimumDistance.resize(kmerCount);
+    for(uint64_t i=0; i<kmerCount; i++) {
+        selectKmers4Data.minimumDistance[i].second = std::numeric_limits<uint32_t>::max();
+    }
+
+    // Compute the minimumDistance vector.
+    setupLoadBalancing(reads->readCount(), 100);
+    runThreads(&Assembler::selectKmers4ThreadFunction, threadCount);
+
+
+
+    // Write out what we found.
+    if(debug) {
+        const uint64_t totalFrequency = std::accumulate(
+            selectKmers4Data.globalFrequency.begin(),
+            selectKmers4Data.globalFrequency.end(), 0ULL);
+        cout << "Total number of k-mer occurrences in all oriented reads is " << totalFrequency << endl;
+        ofstream csv("KmerInfo.csv");
+        csv << "KmerId,Kmer,KmerIdRc,KmerRc,Frequency,FrequencyRc,TotalFrequency,"
+            "MinDist,MinDistRc,MinMinDist\n";
+        for(uint64_t kmerId=0; kmerId<kmerTable.size(); kmerId++) {
+            const KmerInfo& info = kmerTable[kmerId];
+            if(!info.isRleKmer) {
+                continue;
+            }
+
+            const uint64_t frequency = selectKmers4Data.globalFrequency[kmerId];
+            const uint64_t frequencyReverseComplement = selectKmers4Data.globalFrequency[info.reverseComplementedKmerId];
+            const uint64_t totalFrequency = frequency + frequencyReverseComplement;
+
+            const uint32_t minimumDistance = selectKmers4Data.minimumDistance[kmerId].second;
+            const uint32_t minimumDistanceReverseComplement =
+                selectKmers4Data.minimumDistance[info.reverseComplementedKmerId].second;
+
+            const Kmer kmer(kmerId, k);
+            const Kmer reverseComplementedKmer(info.reverseComplementedKmerId, k);
+            csv << kmerId << ",";
+            kmer.write(csv, k);
+            csv << ",";
+            csv << info.reverseComplementedKmerId << ",";
+            reverseComplementedKmer.write(csv, k);
+            csv << ",";
+            csv << frequency << ",";
+            csv << frequencyReverseComplement << ",";
+            csv << totalFrequency << ",";
+            csv << minimumDistance << ",";
+            csv << minimumDistanceReverseComplement << ",";
+            csv << min(minimumDistance, minimumDistanceReverseComplement) << "\n";
+        }
+    }
+
+
+
+    // Compute the total number of k-mer occurrences
+    // and the number of RLE kmers.
+    uint64_t totalKmerOccurrences = 0;
+    uint64_t rleKmerCount = 0;
+    for(uint64_t kmerId=0; kmerId!=kmerTable.size(); kmerId++) {
+        const KmerInfo& info = kmerTable[kmerId];
+        if(not info.isRleKmer) {
+            SHASTA_ASSERT(selectKmers4Data.globalFrequency[kmerId] == 0);
+            continue;
+        }
+        totalKmerOccurrences += selectKmers4Data.globalFrequency[kmerId];
+        if(kmerTable[kmerId].isRleKmer) {
+            ++rleKmerCount;
+        }
+    }
+    cout << "K-mer length k " << k << endl;
+    cout << "Distance threshold " << distanceThreshold << " RLE bases." << endl;
+    cout << "Total number of distinct RLE k-mers " << rleKmerCount << endl;
+    cout << "Total number of RLE k-mers in all oriented reads " << totalKmerOccurrences << endl;
+    cout << "Requested marker density " << markerDensity << endl;
+    const uint64_t requiredMarkerOccurrences = uint64_t(markerDensity * double(totalKmerOccurrences));
+    cout << "Required number of marker occurrences in all oriented reads " << requiredMarkerOccurrences << endl;
+
+
+
+    // Gather k-mers for which the minimum distance between two copies
+    // equals at least distanceThreshold. Exclude palindromic k-mers.
+    vector<KmerId> candidateKmers;
+    uint64_t candidateFrequency = 0;
+    for(uint64_t kmerId=0; kmerId<kmerTable.size(); kmerId++) {
+        const KmerInfo& info = kmerTable[kmerId];
+        const KmerId kmerIdRc = info.reverseComplementedKmerId;
+        if(not info.isRleKmer) {
+            continue;
+        }
+        if(kmerIdRc == kmerId) {
+            // Palindromic. Exclude.
+            continue;
+        }
+        // Only store the lower KmerId in the pair.
+        if(kmerId > kmerIdRc) {
+            continue;
+        }
+        if(selectKmers4Data.minimumDistance[kmerId].second < distanceThreshold) {
+            // Too close. skip.
+            continue;
+        }
+        if(selectKmers4Data.minimumDistance[kmerIdRc].second < distanceThreshold) {
+            // Too close. Skip.
+            continue;
+        }
+
+        candidateKmers.push_back(KmerId(kmerId));
+        candidateFrequency += selectKmers4Data.globalFrequency[kmerId];
+        candidateFrequency += selectKmers4Data.globalFrequency[kmerIdRc];
+    }
+    cout << "Markers will be chosen randomly from the a pool of " <<
+        2*candidateKmers.size() << " RLE k-mers." << endl;
+    cout << "RLE k-mers in this pool occur " <<
+        candidateFrequency << " times in all oriented reads." << endl;
+    cout << "This is sufficient to achieve marker density up to " <<
+        double(candidateFrequency) / double(totalKmerOccurrences) << endl;
+
+    // If these candidates don't have sufficient frequency, we
+    // can't achieve the required marker density.
+    if(candidateFrequency < requiredMarkerOccurrences) {
+        throw runtime_error("Cannot achieve required marker density. "
+            "Increase k, or decrease marker density, or decrease distance threshold.");
+    }
+
+    // Flag all k-mers as not markers.
+    for(uint64_t kmerId=0; kmerId<kmerTable.size(); kmerId++) {
+        kmerTable[kmerId].isMarker = false;
+    }
+
+
+
+    // Randomly pick markers in this vector of candidate k-mers.
+    std::mt19937_64 randomSource(seed);
+    std::uniform_real_distribution<> uniformDistribution;
+    uint64_t markerOccurrencesCount = 0;
+    uint64_t markerCount = 0;
+    while(true) {
+
+        // Pick a random index in the candidateKmers vector.
+        const double x = uniformDistribution(randomSource);  // In [0,1)
+        const uint64_t i = uint64_t(x * double(candidateKmers.size()));
+        SHASTA_ASSERT(i < candidateKmers.size());
+
+        // This KmerId and its reverse complement  will be used as markers.
+        const KmerId kmerId = candidateKmers[i];
+        const KmerId kmerIdRc = kmerTable[kmerId].reverseComplementedKmerId;
+
+        kmerTable[kmerId].isMarker = true;
+        kmerTable[kmerIdRc].isMarker = true;
+
+        // Increment counters.
+        markerCount += 2;
+        markerOccurrencesCount += selectKmers4Data.globalFrequency[kmerId];
+        markerOccurrencesCount += selectKmers4Data.globalFrequency[kmerIdRc];
+
+        // Remove kmerId from the vector of candidates.
+        if(i != candidateKmers.size()-1) {
+            candidateKmers[i] = candidateKmers.back();
+        }
+        candidateKmers.resize(candidateKmers.size() - 1);
+
+        if(markerOccurrencesCount >= requiredMarkerOccurrences) {
+            break;
+        }
+    }
+    cout << "Selected " << markerCount << " k-mers as markers." << endl;
+    cout << "Actual marker density " << double(markerOccurrencesCount) / double(totalKmerOccurrences) << endl;
+
+
+
+    // Clean up.
+    selectKmers4Data.minimumDistance.remove();
+    selectKmers4Data.globalFrequency.remove();
+
+    // Done.
+    cout << timestamp << "End selectKmers4." << endl;
+}
+
+
+
+void Assembler::selectKmers4ThreadFunction(size_t threadId)
+{
+    // K-mer length.
+    const size_t k = assemblerInfo->k;
+
+    // Initialize globalFrequency for this thread.
+    // Having all threads accumulate atomically on the global frequency vector is too slow.
+    MemoryMapped::Vector<uint64_t> globalFrequency;
+    globalFrequency.createNew(
+        largeDataName("tmp-SelectKmers4-GlobalFrequency-" + to_string(threadId)),
+        largeDataPageSize);
+    globalFrequency.resize(kmerTable.size());
+    fill(globalFrequency.begin(), globalFrequency.end(), 0);
+
+    // Vector to hold pairs(KmerId, RLE position) for one read.
+    vector< pair<KmerId, uint32_t> > readKmers;
+
+    // Loop over all batches assigned to this thread.
+    uint64_t begin, end;
+    while(getNextBatch(begin, end)) {
+
+        // Loop over all reads assigned to this batch.
+        for(ReadId readId=ReadId(begin); readId!=ReadId(end); readId++) {
+
+            // Access the sequence of this read.
+            const LongBaseSequenceView read = reads->getRead(readId);
+
+            // If the read is pathologically short, it has no k-mers.
+            if(read.baseCount < k) {
+                continue;
+            }
+
+            // Loop over k-mers of this read.
+            Kmer kmer;
+            for(size_t position=0; position<k; position++) {
+                kmer.set(position, read[position]);
+            }
+            readKmers.clear();
+            for(uint32_t position=0; /*The check is done later */; position++) {
+
+                // Get the k-mer id.
+                const KmerId kmerId = KmerId(kmer.id(k));
+                readKmers.push_back(make_pair(kmerId, position));
+
+                // Update the frequency of this k-mer.
+                ++globalFrequency[kmerId];
+                ++globalFrequency[kmerTable[kmerId].reverseComplementedKmerId];
+
+                // Check if we reached the end of the read.
+                if(position+k == read.baseCount) {
+                    break;
+                }
+
+                // Update the k-mer.
+                kmer.shiftLeft();
+                kmer.set(k-1, read[position+k]);
+            }
+
+            // Sort by k-mer, then by position.
+            sort(readKmers.begin(), readKmers.end());
+
+            // Update minDistance for each pair of repeated k-mers.
+            for(uint64_t i=1; i<readKmers.size(); i++) {
+                const auto& p0 = readKmers[i-1];
+                const auto& p1 = readKmers[i];
+                const KmerId kmerId0 = p0.first;
+                const KmerId kmerId1 = p1.first;
+                if(kmerId0 != kmerId1) {
+                    continue;
+                }
+                const uint32_t distance = p1.second - p0.second;
+
+                pair<std::mutex, uint32_t>& p = selectKmers4Data.minimumDistance[kmerId0];
+                std::lock_guard<std::mutex> lock(p.first);;
+                p.second = min(p.second, distance);
+            }
+        }
+    }
+
+    // Add our globalFrequency to the values computed by the other threads.
+    {
+        std::lock_guard<std::mutex> lock(mutex);
+        for(uint64_t kmerId=0; kmerId!=globalFrequency.size(); kmerId++) {
+            selectKmers4Data.globalFrequency[kmerId] += globalFrequency[kmerId];
+        }
+    }
+    globalFrequency.remove();
+}
+
