@@ -1,4 +1,5 @@
 #include "AssemblerOptions.hpp"
+#include "ConfigurationTable.hpp"
 #include "SimpleBayesianConsensusCaller.hpp"
 #include "buildId.hpp"
 #include "filesystem.hpp"
@@ -9,7 +10,7 @@ using namespace shasta;
 
 // Standard library.
 #include "fstream.hpp"
-#include"stdexcept.hpp"
+#include "stdexcept.hpp"
 
 
 
@@ -79,17 +80,42 @@ AssemblerOptions::AssemblerOptions(int argumentCount, const char** arguments) :
         ::exit(0);
     }
 
-    // Get options from the config file, if one was specified.
-    if(!commandLineOnlyOptions.configFileName.empty()) {
-        ifstream configFile(commandLineOnlyOptions.configFileName);
-        if (!configFile) {
-            throw runtime_error("Unable to open config file " +
-                commandLineOnlyOptions.configFileName);
+
+
+    // Get options from the config file or built-in configuration, if one was specified.
+    if(!commandLineOnlyOptions.configName.empty()) {
+
+        // Look it up in our configuration table.
+        const string* builtInConfiguration = getConfiguration(commandLineOnlyOptions.configName);
+
+        if(builtInConfiguration) {
+
+            // --config specifies a built-in configuration. Get options from that.
+            std::istringstream s(*builtInConfiguration);
+            store(parse_config_file(s, configurableOptionsDescription), variablesMap);
+            notify(variablesMap);
+
+        } else {
+
+            // See if --config specifies a configuration file.
+            ifstream configFile(commandLineOnlyOptions.configName);
+            if (!configFile) {
+                string message =
+                    "The --config option does not specify a "
+                    "valid built-in configuration or configuration file. "
+                    "Valid built-in configurations are the following:\n";
+                for(const auto& p: configurationTable) {
+                    message += p.first + "\n";
+                }
+                throw runtime_error(message);
+            }
+            store(parse_config_file(configFile, configurableOptionsDescription), variablesMap);
+            notify(variablesMap);
         }
-        store(parse_config_file(configFile, configurableOptionsDescription), variablesMap);
-        notify(variablesMap);
     }
 
+
+
     // Parse MarkerGraph.simplifyMaxLength.
     markerGraphOptions.parseSimplifyMaxLength();
 
@@ -120,8 +146,9 @@ void AssemblerOptions::addCommandLineOnlyOptions()
         "Identify the Shasta version.")
 
         ("config",
-        value<string>(&commandLineOnlyOptions.configFileName),
-        "Configuration file name.")
+        value<string>(&commandLineOnlyOptions.configName),
+        "Configuration name. Can be the name of a built-in configuration "
+        "or the name of a configuration file.")
 
         ("input",
         value< vector<string> >(&commandLineOnlyOptions.inputFileNames)->multitoken(),
@@ -175,6 +202,13 @@ void AssemblerOptions::addCommandLineOnlyOptions()
         value<uint16_t>(&commandLineOnlyOptions.port)->
         default_value(17100),
         "Port to be used by the http server (command --explore).")
+
+        ("alignmentsPafFile",
+        value<string>(&commandLineOnlyOptions.alignmentsPafFile),
+        "The name of a PAF file containing alignments of reads to "
+        "a reference. Only used for --command explore, for display of the alignment "
+        "candidate graph. Experimental."
+        )
 #endif
         ;
 
@@ -261,14 +295,39 @@ void AssemblerOptions::addConfigurableOptions()
          default_value(100),
          "Used for palindromic read detection.")
 
+        ("Reads.palindromicReads.detectOnFastqLoad",
+         bool_switch(&readsOptions.palindromicReads.detectOnFastqLoad)->
+         default_value(false),
+        "Filter reads that have exceptionally poor quality in the second half, "
+        "which is a strong indication of palindromic sequence.")
+
+        ("Reads.palindromicReads.qScoreRelativeMeanDifference",
+        value<double>(&readsOptions.palindromicReads.qScoreRelativeMeanDifference)->
+        default_value(0.09, "0.09"),
+        "When filtering palindrome quality, this parameter describes how much "
+        "worse should the right side average p(error) should be.")
+
+        ("Reads.palindromicReads.qScoreMinimumMean",
+        value<double>(&readsOptions.palindromicReads.qScoreMinimumMean)->
+        default_value(0.15, "0.15"),
+        "When filtering palindrome quality, this parameter describes the absolute "
+        "minimum average p(error) in the right half.")
+
+        ("Reads.palindromicReads.qScoreMinimumVariance",
+         value<double>(&readsOptions.palindromicReads.qScoreMinimumVariance)->
+         default_value(0.025, "0.025"),
+         "When filtering palindrome quality, this parameter describes the absolute "
+         "minimum variance in the right half.")
+
          ("Kmers.generationMethod",
          value<int>(&kmersOptions.generationMethod)->
          default_value(0),
          "Method to generate marker k-mers: "
          "0 = random, "
-         "1 = random, excluding globally overenriched,"
-         "2 = random, excluding overenriched even in a single read,"
-         "3 = read from file.")
+         "1 = random, excluding globally overenriched k-mers,"
+         "2 = random, excluding k-mers overenriched even in a single read,"
+         "3 = read from file."
+         "4 = random, excluding k-mers appearing in two copies close to each other even in a single read.")
 
          ("Kmers.k",
          value<int>(&kmersOptions.k)->
@@ -285,6 +344,11 @@ void AssemblerOptions::addConfigurableOptions()
         default_value(100., "100."),
         "Enrichment threshold for Kmers.generationMethod 1 and 2.")
 
+        ("Kmers.distanceThreshold",
+        value<uint64_t>(&kmersOptions.distanceThreshold)->
+        default_value(1000),
+        "Distance threshold, in RLE bases, for Kmers.generationMethod 4")
+
         ("Kmers.file",
         value<string>(&kmersOptions.file),
         "The absolute path of a file containing the k-mers "
@@ -325,12 +389,12 @@ void AssemblerOptions::addConfigurableOptions()
         ("MinHash.minBucketSize",
         value<int>(&minHashOptions.minBucketSize)->
         default_value(0),
-        "The minimum bucket size to be used by the LowHash algoritm.")
+        "The minimum bucket size to be used by the LowHash algorithm.")
 
         ("MinHash.maxBucketSize",
         value<int>(&minHashOptions.maxBucketSize)->
         default_value(10),
-        "The maximum bucket size to be used by the LowHash algoritm.")
+        "The maximum bucket size to be used by the LowHash algorithm.")
 
         ("MinHash.minFrequency",
         value<int>(&minHashOptions.minFrequency)->
@@ -349,7 +413,7 @@ void AssemblerOptions::addConfigurableOptions()
         value<int>(&alignOptions.alignMethod)->
         default_value(3),
         "The alignment method to be used to create the read graph & the marker graph. "
-        "0 = old Shasta method, 1 = SeqAn (slow), 3 = banded SeqAn.")
+        "0 = old Shasta method, 1 = SeqAn (slow), 3 = banded SeqAn, 4 = new Shasta method (experimental).")
 
         ("Align.maxSkip",
         value<int>(&alignOptions.maxSkip)->
@@ -436,6 +500,26 @@ void AssemblerOptions::addConfigurableOptions()
         "one read is entirely contained in another read, "
         "except possibly for up to maxTrim markers at the beginning and end.")
 
+        ("Align.align4.deltaX",
+        value<uint64_t>(&alignOptions.align4DeltaX)->
+        default_value(200),
+        "Only used for alignment method 4 (experimental).")
+
+        ("Align.align4.deltaY",
+        value<uint64_t>(&alignOptions.align4DeltaY)->
+        default_value(10),
+        "Only used for alignment method 4 (experimental).")
+
+        ("Align.align4.minEntryCountPerCell",
+        value<uint64_t>(&alignOptions.align4MinEntryCountPerCell)->
+        default_value(10),
+        "Only used for alignment method 4 (experimental).")
+
+        ("Align.align4.maxDistanceFromBoundary",
+        value<uint64_t>(&alignOptions.align4MaxDistanceFromBoundary)->
+        default_value(100),
+        "Only used for alignment method 4 (experimental).")
+
         ("ReadGraph.creationMethod",
         value<int>(&readGraphOptions.creationMethod)->
         default_value(0),
@@ -446,34 +530,23 @@ void AssemblerOptions::addConfigurableOptions()
         default_value(6),
         "The maximum number of alignments to be kept for each read.")
 
-        ("ReadGraph.minComponentSize",
-        value<int>(&readGraphOptions.minComponentSize)->
-        default_value(100),
-        "The minimum size (number of oriented reads) of a connected component "
-        "of the read graph to be kept. This is currently ignored.")
-
         ("ReadGraph.maxChimericReadDistance",
         value<int>(&readGraphOptions.maxChimericReadDistance)->
         default_value(2),
         "Used for chimeric read detection.")
 
+        ("ReadGraph.strandSeparationMethod",
+        value<uint64_t>(&readGraphOptions.strandSeparationMethod)->
+        default_value(1),
+        "Strand separation method: "
+        "0 = no strand separation, "
+        "1 = limited strand separation, "
+        "2 = strict strand separation.")
+
         ("ReadGraph.crossStrandMaxDistance",
         value<int>(&readGraphOptions.crossStrandMaxDistance)->
         default_value(6),
-        "Maximum distance (edges) for flagCrossStrandReadGraphEdges. "
-        "Set this to zero to entirely suppress flagCrossStrandReadGraphEdges.")
-
-        ("ReadGraph.containedNeighborCount",
-        value<int>(&readGraphOptions.containedNeighborCount)->
-        default_value(6),
-        "Maximum number of alignments to be kept for each contained read "
-        "(only used when creationMethod is 1).")
-
-        ("ReadGraph.uncontainedNeighborCountPerDirection",
-        value<int>(&readGraphOptions.uncontainedNeighborCountPerDirection)->
-        default_value(3),
-        "Maximum number of alignments to be kept in each direction "
-        "(forward, backward) for each uncontained read (only used when creationMethod is 1).")
+        "Maximum distance (edges) for strand separation method 1.")
 
         ("ReadGraph.removeConflicts",
         bool_switch(&readGraphOptions.removeConflicts)->
@@ -483,27 +556,55 @@ void AssemblerOptions::addConfigurableOptions()
         ("ReadGraph.markerCountPercentile",
         value<double>(&readGraphOptions.markerCountPercentile)->
         default_value(0.015, "0.015"),
-        "Percentile for markerCount (only used when creationMethod is 2).")
+        "Percentile for --ReadGraph.markerCount "
+        "(only used when --ReadGraph.creationMethod is 2).")
 
         ("ReadGraph.alignedFractionPercentile",
         value<double>(&readGraphOptions.alignedFractionPercentile)->
         default_value(0.12, "0.12"),
-        "Percentile for alignedFraction (only used when creationMethod is 2).")
+        "Percentile for adaptive selection of --ReadGraph.alignedFraction "
+        "(only used when --ReadGraph.creationMethod is 2).")
 
         ("ReadGraph.maxSkipPercentile",
         value<double>(&readGraphOptions.maxSkipPercentile)->
         default_value(0.12, "0.12"),
-        "Percentile for maxSkip (only used when creationMethod is 2).")
+        "Percentile for adaptive selection of --ReadGraph.maxSkip "
+        "(only used when --ReadGraph.creationMethod is 2).")
 
         ("ReadGraph.maxDriftPercentile",
         value<double>(&readGraphOptions.maxDriftPercentile)->
         default_value(0.12, "0.12"),
-        "Percentile for maxDrift (only used when creationMethod is 2).")
+        "Percentile for adaptive selection of --ReadGraph.maxDrift "
+        "(only used when --ReadGraph.creationMethod is 2).")
 
         ("ReadGraph.maxTrimPercentile",
         value<double>(&readGraphOptions.maxTrimPercentile)->
         default_value(0.015, "0.015"),
-        "Percentile for maxTrim (only used when creationMethod is 2).")
+        "Percentile for adaptive selection of --ReadGraph.maxTrim "
+        "(only used when --ReadGraph.creationMethod is 2).")
+
+        ("ReadGraph.flagInconsistentAlignments",
+        bool_switch(&readGraphOptions.flagInconsistentAlignments)->
+        default_value(false),
+        "Flag inconsistent alignments. Experimental.")
+
+        ("ReadGraph.flagInconsistentAlignments.triangleErrorThreshold",
+        value<uint64_t>(&readGraphOptions.flagInconsistentAlignmentsTriangleErrorThreshold)->
+        default_value(200),
+        "Triangle error threshold, in markers, for flagging inconsistent alignments. "
+        "Only used if --ReadGraph.flagInconsistentAlignments is set. Experimental.")
+
+        ("ReadGraph.flagInconsistentAlignments.leastSquareErrorThreshold",
+        value<uint64_t>(&readGraphOptions.flagInconsistentAlignmentsLeastSquareErrorThreshold)->
+        default_value(200),
+        "Least square error threshold, in markers, for flagging inconsistent alignments. "
+        "Only used if --ReadGraph.flagInconsistentAlignments is set. Experimental.")
+
+        ("ReadGraph.flagInconsistentAlignments.leastSquareMaxDistance",
+        value<uint64_t>(&readGraphOptions.flagInconsistentAlignmentsLeastSquareMaxDistance)->
+        default_value(1),
+        "Least square max distance for flagging inconsistent alignments. "
+        "Only used if --ReadGraph.flagInconsistentAlignments is set. Experimental.")
 
         ("MarkerGraph.minCoverage",
         value<int>(&markerGraphOptions.minCoverage)->
@@ -525,6 +626,39 @@ void AssemblerOptions::addConfigurableOptions()
         "Minimum coverage (number of supporting oriented reads) "
         "for each strand for a marker graph vertex.")
 
+        ("MarkerGraph.minEdgeCoverage",
+        value<uint64_t>(&markerGraphOptions.minEdgeCoverage)->
+        default_value(6),
+        "Minimum edge coverage (number of supporting oriented reads) "
+        "for a marker graph edge to be created."
+        "Experimental. Only used with --Assembly.mode 1.")
+
+        ("MarkerGraph.minEdgeCoveragePerStrand",
+        value<uint64_t>(&markerGraphOptions.minEdgeCoveragePerStrand)->
+        default_value(2),
+        "Minimum edge coverage (number of supporting oriented reads) "
+        "on each strand "
+        "for a marker graph edge to be created."
+        "Experimental. Only used with --Assembly.mode 1.")
+
+        ("MarkerGraph.allowDuplicateMarkers",
+        bool_switch(&markerGraphOptions.allowDuplicateMarkers)->
+        default_value(false),
+        "Specifies whether to allow more than one marker on the "
+        "same oriented read on a single marker graph vertex. Experimental.")
+
+        ("MarkerGraph.cleanupDuplicateMarkers",
+        bool_switch(&markerGraphOptions.cleanupDuplicateMarkers)->
+        default_value(false),
+        "Specifies whether to clean up marker graph vertices with more than one marker on the "
+        "same oriented read. Experimental.")
+
+        ("MarkerGraph.duplicateMarkersPattern1Threshold",
+        value<double>(&markerGraphOptions.duplicateMarkersPattern1Threshold)->
+        default_value(0.5),
+        "Used when cleaning up marker graph vertices with more than one marker on the "
+        "same oriented read. Experimental.")
+
         ("MarkerGraph.lowCoverageThreshold",
         value<int>(&markerGraphOptions.lowCoverageThreshold)->
         default_value(0),
@@ -588,6 +722,11 @@ void AssemblerOptions::addConfigurableOptions()
         "Used in the automatic selection of --MarkerGraph.minCoverage when "
         "--MarkerGraph.minCoverage is set to 0.")
 
+        ("Assembly.mode",
+        value<uint64_t>(&assemblyOptions.mode)->
+        default_value(0),
+        "Assembly mode (0=default, 1=experimental).")
+
         ("Assembly.crossEdgeCoverageThreshold",
         value<int>(&assemblyOptions.crossEdgeCoverageThreshold)->
         default_value(3),
@@ -601,7 +740,7 @@ void AssemblerOptions::addConfigurableOptions()
 
         ("Assembly.consensusCaller",
         value<string>(&assemblyOptions.consensusCallerString)->
-        default_value("Bayesian:guppy-2.3.5-a"),
+        default_value("Modal"),
         "Selects the consensus caller for repeat counts. "
         "See the documentation for available choices.")
 
@@ -704,12 +843,64 @@ void AssemblerOptions::addConfigurableOptions()
         value<uint64_t>(&assemblyOptions.iterativeBridgeRemovalMaxDistance)->
         default_value(2),
         "Maximum distance for read graph bridge removal for iterative assembly (experimental).")
+
+        ("Assembly.bubbleRemoval.discordantRatioThreshold",
+        value<double>(&assemblyOptions.bubbleRemovalDiscordantRatioThreshold)->
+        default_value(0.2),
+        "Discordant ratio threshold for bubble removal (assembly mode 2 only, experimental).")
+
+        ("Assembly.bubbleRemoval.ambiguityThreshold",
+        value<double>(&assemblyOptions.bubbleRemovalAmbiguityThreshold)->
+        default_value(0.5),
+        "Ambiguity threshold for bubble removal (assembly mode 2 only, experimental).")
+
+        ("Assembly.bubbleRemoval.maxPeriod",
+        value<uint64_t>(&assemblyOptions.bubbleRemovalMaxPeriod)->
+        default_value(4),
+        "Maximum repeat period for bubble removal (assembly mode 2 only, experimental).")
+
+        ("Assembly.superbubbleRemoval.edgeLengthThreshold",
+        value<uint64_t>(&assemblyOptions.superbubbleRemovalEdgeLengthThreshold)->
+        default_value(6),
+        "Edge length threshold in markers for superbubble removal "
+        "(assembly mode 2 only, experimental).")
+
+        ("Assembly.phasing.minReadCount",
+        value<uint64_t>(&assemblyOptions.phasingMinReadCount)->
+        default_value(3),
+        "Minimum number of reads for phasing (assembly mode 2 only, experimental).")
+
+        ("Assembly.suppressGfaOutput",
+        bool_switch(&assemblyOptions.suppressGfaOutput)->
+        default_value(false),
+        "Suppress all GFA output (Mode 2 assembly only).")
+
+        ("Assembly.suppressFastaOutput",
+        bool_switch(&assemblyOptions.suppressFastaOutput)->
+        default_value(false),
+        "Suppress all FASTA output (Mode 2 assembly only).")
+
+        ("Assembly.suppressDetailedOutput",
+        bool_switch(&assemblyOptions.suppressDetailedOutput)->
+        default_value(false),
+        "Suppress output of detailed representation of the assembly (Mode 2 assembly only).")
+
+        ("Assembly.suppressPhasedOutput",
+        bool_switch(&assemblyOptions.suppressPhasedOutput)->
+        default_value(false),
+        "Suppress output of phased representation of the assembly (Mode 2 assembly only).")
+
+        ("Assembly.suppressHaploidOutput",
+        bool_switch(&assemblyOptions.suppressHaploidOutput)->
+        default_value(false),
+        "Suppress output of haploid representation of the assembly (Mode 2 assembly only).")
+
         ;
 }
 
 
 
-void AssemblerOptions::ReadsOptions::PalindromicReadOptions::write(ostream& s) const
+void PalindromicReadOptions::write(ostream& s) const
 {
     s << "palindromicReads.skipFlagging = " << convertBoolToPythonString(skipFlagging) << "\n";
     s << "palindromicReads.maxSkip = " << maxSkip << "\n";
@@ -718,11 +909,14 @@ void AssemblerOptions::ReadsOptions::PalindromicReadOptions::write(ostream& s) c
     s << "palindromicReads.alignedFractionThreshold = " << alignedFractionThreshold << "\n";
     s << "palindromicReads.nearDiagonalFractionThreshold = " << nearDiagonalFractionThreshold << "\n";
     s << "palindromicReads.deltaThreshold = " << deltaThreshold << "\n";
+    s << "palindromicReads.qScoreRelativeMeanDifference = " << qScoreRelativeMeanDifference << "\n";
+    s << "palindromicReads.qScoreMinimumMean = " << qScoreMinimumMean << "\n";
+    s << "palindromicReads.qScoreMinimumVariance = " << qScoreMinimumVariance << "\n";
 }
 
 
 
-void AssemblerOptions::ReadsOptions::write(ostream& s) const
+void ReadsOptions::write(ostream& s) const
 {
     s << "[Reads]\n";
     s << "minReadLength = " << minReadLength << "\n";
@@ -734,19 +928,20 @@ void AssemblerOptions::ReadsOptions::write(ostream& s) const
 
 
 
-void AssemblerOptions::KmersOptions::write(ostream& s) const
+void KmersOptions::write(ostream& s) const
 {
     s << "[Kmers]\n";
     s << "generationMethod = " << generationMethod << "\n";
     s << "k = " << k << "\n";
     s << "probability = " << probability << "\n";
     s << "enrichmentThreshold = " << enrichmentThreshold << "\n";
+    s << "distanceThreshold = " << distanceThreshold << "\n";
     s << "file = " << file << "\n";
 }
 
 
 
-void AssemblerOptions::MinHashOptions::write(ostream& s) const
+void MinHashOptions::write(ostream& s) const
 {
     s << "[MinHash]\n";
     s << "version = " << version << "\n";
@@ -763,7 +958,7 @@ void AssemblerOptions::MinHashOptions::write(ostream& s) const
 
 
 
-void AssemblerOptions::AlignOptions::write(ostream& s) const
+void AlignOptions::write(ostream& s) const
 {
     s << "[Align]\n";
     s << "alignMethod = " << alignMethod << "\n";
@@ -783,20 +978,22 @@ void AssemblerOptions::AlignOptions::write(ostream& s) const
         sameChannelReadAlignmentSuppressDeltaThreshold << "\n";
     s << "suppressContainments = " <<
         convertBoolToPythonString(suppressContainments) << "\n";
+    s << "align4.deltaX = " << align4DeltaX << "\n";
+    s << "align4.deltaY = " << align4DeltaY << "\n";
+    s << "align4.minEntryCountPerCell = " << align4MinEntryCountPerCell << "\n";
+    s << "align4.maxDistanceFromBoundary = " << align4MaxDistanceFromBoundary << "\n";
 }
 
 
 
-void AssemblerOptions::ReadGraphOptions::write(ostream& s) const
+void ReadGraphOptions::write(ostream& s) const
 {
     s << "[ReadGraph]\n";
     s << "creationMethod = " << creationMethod << "\n";
     s << "maxAlignmentCount = " << maxAlignmentCount << "\n";
-    s << "minComponentSize = " << minComponentSize << "\n";
     s << "maxChimericReadDistance = " << maxChimericReadDistance << "\n";
+    s << "strandSeparationMethod = " << strandSeparationMethod << "\n";
     s << "crossStrandMaxDistance = " << crossStrandMaxDistance << "\n";
-    s << "containedNeighborCount = " << containedNeighborCount << "\n";
-    s << "uncontainedNeighborCountPerDirection = " << uncontainedNeighborCountPerDirection << "\n";
     s << "removeConflicts = " <<
         convertBoolToPythonString(removeConflicts) << "\n";
     s << "markerCountPercentile = " << markerCountPercentile << "\n";
@@ -804,16 +1001,31 @@ void AssemblerOptions::ReadGraphOptions::write(ostream& s) const
     s << "maxSkipPercentile = " << maxSkipPercentile << "\n";
     s << "maxDriftPercentile = " << maxDriftPercentile << "\n";
     s << "maxTrimPercentile = " << maxTrimPercentile << "\n";
+
+    s << "flagInconsistentAlignments = " << convertBoolToPythonString(flagInconsistentAlignments) << "\n";
+    s << "flagInconsistentAlignments.triangleErrorThreshold = " <<
+        flagInconsistentAlignmentsTriangleErrorThreshold << "\n";
+    s << "flagInconsistentAlignments.leastSquareErrorThreshold = " <<
+        flagInconsistentAlignmentsLeastSquareErrorThreshold << "\n";
+    s << "flagInconsistentAlignments.leastSquareMaxDistance = " <<
+        flagInconsistentAlignmentsLeastSquareMaxDistance << "\n";
 }
 
 
 
-void AssemblerOptions::MarkerGraphOptions::write(ostream& s) const
+void MarkerGraphOptions::write(ostream& s) const
 {
     s << "[MarkerGraph]\n";
     s << "minCoverage = " << minCoverage << "\n";
     s << "maxCoverage = " << maxCoverage << "\n";
     s << "minCoveragePerStrand = " << minCoveragePerStrand << "\n";
+    s << "minEdgeCoverage = " << minEdgeCoverage << "\n";
+    s << "minEdgeCoveragePerStrand = " << minEdgeCoveragePerStrand << "\n";
+    s << "allowDuplicateMarkers = " <<
+        convertBoolToPythonString(allowDuplicateMarkers) << "\n";
+    s << "cleanupDuplicateMarkers = " <<
+        convertBoolToPythonString(cleanupDuplicateMarkers) << "\n";
+    s << "duplicateMarkersPattern1Threshold = " << duplicateMarkersPattern1Threshold << "\n";
     s << "lowCoverageThreshold = " << lowCoverageThreshold << "\n";
     s << "highCoverageThreshold = " << highCoverageThreshold << "\n";
     s << "maxDistance = " << maxDistance << "\n";
@@ -830,9 +1042,10 @@ void AssemblerOptions::MarkerGraphOptions::write(ostream& s) const
 
 
 
-void AssemblerOptions::AssemblyOptions::write(ostream& s) const
+void AssemblyOptions::write(ostream& s) const
 {
     s << "[Assembly]\n";
+    s << "mode = " << mode << "\n";
     s << "crossEdgeCoverageThreshold = " << crossEdgeCoverageThreshold << "\n";
     s << "markerGraphEdgeLengthThresholdForConsensus = " <<
         markerGraphEdgeLengthThresholdForConsensus << "\n";
@@ -859,6 +1072,18 @@ void AssemblerOptions::AssemblyOptions::write(ostream& s) const
     s << "iterative.maxAlignmentCount = " << iterativeMaxAlignmentCount << "\n";
     s << "iterative.bridgeRemovalIterationCount = " << iterativeBridgeRemovalIterationCount << "\n";
     s << "iterative.bridgeRemovalMaxDistance = " << iterativeBridgeRemovalMaxDistance << "\n";
+
+    s << "bubbleRemoval.discordantRatioThreshold = " << bubbleRemovalDiscordantRatioThreshold << "\n";
+    s << "bubbleRemoval.ambiguityThreshold = " << bubbleRemovalAmbiguityThreshold << "\n";
+    s << "bubbleRemoval.maxPeriod = " << bubbleRemovalMaxPeriod << "\n";
+    s << "superbubbleRemoval.edgeLengthThreshold = " << superbubbleRemovalEdgeLengthThreshold << "\n";
+    s << "phasing.minReadCount = " << phasingMinReadCount << "\n";
+
+    s << "suppressGfaOutput = " << convertBoolToPythonString(suppressGfaOutput) << "\n";
+    s << "suppressFastaOutput = " << convertBoolToPythonString(suppressFastaOutput) << "\n";
+    s << "suppressDetailedOutput = " << convertBoolToPythonString(suppressDetailedOutput) << "\n";
+    s << "suppressPhasedOutput = " << convertBoolToPythonString(suppressPhasedOutput) << "\n";
+    s << "suppressHaploidOutput = " << convertBoolToPythonString(suppressHaploidOutput) << "\n";
 }
 
 
@@ -883,7 +1108,7 @@ void AssemblerOptions::write(ostream& s) const
 
 
 
-void AssemblerOptions::MarkerGraphOptions::parseSimplifyMaxLength()
+void MarkerGraphOptions::parseSimplifyMaxLength()
 {
     simplifyMaxLengthVector.clear();
 
@@ -906,7 +1131,9 @@ void AssemblerOptions::MarkerGraphOptions::parseSimplifyMaxLength()
 
 }
 
-void AssemblerOptions::ReadsOptions::parseDesiredCoverageString() {
+
+
+void ReadsOptions::parseDesiredCoverageString() {
     size_t pos = 0;
     desiredCoverage = std::stoull(desiredCoverageString, &pos);
     
@@ -927,7 +1154,7 @@ void AssemblerOptions::ReadsOptions::parseDesiredCoverageString() {
     }
 }
 
-void AssemblerOptions::AssemblyOptions::parseConsensusCallerString() {
+void AssemblyOptions::parseConsensusCallerString() {
     const size_t colonPos = consensusCallerString.find_first_of(':');
     if (string::npos == colonPos) {
         consensusCaller = consensusCallerString;
@@ -958,9 +1185,11 @@ void AssemblerOptions::AssemblyOptions::parseConsensusCallerString() {
     return;
 }
 
+
+
 // Function to convert a bool to True or False for better
 // compatibility with Python scripts.
-string AssemblerOptions::convertBoolToPythonString(bool flag)
+string shasta::convertBoolToPythonString(bool flag)
 {
     return flag ? "True" : "False";
 }
