@@ -14,6 +14,11 @@ void Assembler::addReads(
     const string& fileName,
     uint64_t minReadLength,
     bool noCache,
+    bool detectPalindromesOnFastqLoad,
+    double qScoreRelativeMeanDifference,
+    double qScoreMinimumMean,
+    double qScoreMinimumVariance,
+    bool writePalindromicReadsToCsv,
     const size_t threadCount)
 {
     reads->checkReadsAreOpen();
@@ -26,6 +31,11 @@ void Assembler::addReads(
         threadCount,
         largeDataFileNamePrefix,
         largeDataPageSize,
+        detectPalindromesOnFastqLoad,
+        qScoreRelativeMeanDifference,
+        qScoreMinimumMean,
+        qScoreMinimumVariance,
+        writePalindromicReadsToCsv,
         *reads);
     
     reads->checkSanity();
@@ -41,6 +51,9 @@ void Assembler::addReads(
     cout << "    Discarded " << readLoader.discardedBadRepeatCountReadCount <<
         " reads containing repeat counts 256 or more" <<
         " for a total " << readLoader.discardedBadRepeatCountBaseCount << " bases." << endl;
+    cout << "    Discarded " << readLoader.discardedPalindromicReadCount <<
+         " reads with palindromic quality scores" <<
+         " for a total " << readLoader.discardedPalindromicBaseCount << " bases." << endl;
 
     // Increment the discarded reads statistics.
     assemblerInfo->discardedInvalidBaseReadCount += readLoader.discardedInvalidBaseReadCount;
@@ -49,6 +62,8 @@ void Assembler::addReads(
     assemblerInfo->discardedShortReadBaseCount += readLoader.discardedShortReadBaseCount;
     assemblerInfo->discardedBadRepeatCountReadCount += readLoader.discardedBadRepeatCountReadCount;
     assemblerInfo->discardedBadRepeatCountBaseCount += readLoader.discardedBadRepeatCountBaseCount;
+    assemblerInfo->discardedPalindromicReadCount += readLoader.discardedPalindromicReadCount;
+    assemblerInfo->discardedPalindromicBaseCount += readLoader.discardedPalindromicBaseCount;
     assemblerInfo->minReadLength = minReadLength;
 }
 
@@ -236,6 +251,7 @@ uint64_t Assembler::adjustCoverageAndGetNewMinReadLength(uint64_t desiredCoverag
         largeDataName("ReadMetaData"),
         largeDataName("ReadRepeatCounts"),
         largeDataName("ReadFlags"),
+        largeDataName("ReadIdsSortedByName"),
         largeDataPageSize
     );
 
@@ -258,3 +274,10 @@ uint64_t Assembler::adjustCoverageAndGetNewMinReadLength(uint64_t desiredCoverag
     return assemblerInfo->minReadLength;
 }
 
+
+
+void Assembler::computeReadIdsSortedByName()
+{
+    reads->computeReadIdsSortedByName();
+}
+
