@@ -5,6 +5,7 @@
 #include "findLinearChains.hpp"
 #include "html.hpp"
 #include "platformDependent.hpp"
+#include "Reads.hpp"
 #include "runCommandWithTimeout.hpp"
 #include "subgraph.hpp"
 using namespace shasta;
@@ -177,7 +178,7 @@ void CompressedAssemblyGraph::mergeLinearChains()
 
     // Find linear chains.
     vector< std::list<edge_descriptor> > chains;
-    findLinearChains(graph, chains);
+    findLinearChains(graph, 0, chains);
 
 
 
@@ -590,7 +591,7 @@ void CompressedAssemblyGraph::writeCsvOrientedReads() const
         const vector<edge_descriptor>& edges = orientedReadTable[orientedReadId];
         for(const edge_descriptor e: edges) {
             const CompressedAssemblyGraphEdge& edge = graph[e];
-            csv << OrientedReadId(orientedReadId) << ",";
+            csv << OrientedReadId::fromValue(orientedReadId) << ",";
             csv << edge.id << ",";
             csv << edge.gfaId() << "\n";
         }
