@@ -17,7 +17,7 @@ void LocalAlignmentGraph::addVertex(
     uint32_t baseCount,
     uint32_t distance)
 {
-    // Check that we don't altready have a vertex with this OrientedReadId.
+    // Check that we don't already have a vertex with this OrientedReadId.
     SHASTA_ASSERT(vertexMap.find(orientedReadId) == vertexMap.end());
 
     // Create the vertex.
@@ -106,7 +106,7 @@ void LocalAlignmentGraph::Writer::operator()(std::ostream& s) const
 void LocalAlignmentGraph::Writer::operator()(std::ostream& s, vertex_descriptor v) const
 {
     const LocalAlignmentGraphVertex& vertex = graph[v];
-    const OrientedReadId orientedReadId(vertex.orientedReadId);
+    const OrientedReadId orientedReadId = OrientedReadId::fromValue(vertex.orientedReadId);
 
     s << "[";
     s << " tooltip=\"" << orientedReadId << " length " << vertex.baseCount << " distance " << vertex.distance << "\"";
@@ -132,8 +132,8 @@ void LocalAlignmentGraph::Writer::operator()(std::ostream& s, edge_descriptor e)
     const LocalAlignmentGraphVertex& vertex1 = graph[v1];
 
     s << "[";
-    s << "tooltip=\"" << OrientedReadId(vertex0.orientedReadId) << " ";
-    s << OrientedReadId(vertex1.orientedReadId) << " ";
+    s << "tooltip=\"" << OrientedReadId::fromValue(vertex0.orientedReadId) << " ";
+    s << OrientedReadId::fromValue(vertex1.orientedReadId) << " ";
     s << edge.alignmentInfo.markerCount << "\"";
     s << "]";
 }
@@ -191,7 +191,7 @@ void LocalAlignmentGraph::writeSvg(
     std::map<vertex_descriptor, VertexAttributes> vertexAttributes;
     BGL_FORALL_VERTICES(v, graph, Graph) {
         const auto& vertex = graph[v];
-        const OrientedReadId orientedReadId = OrientedReadId(vertex.orientedReadId);
+        const OrientedReadId orientedReadId = OrientedReadId::fromValue(vertex.orientedReadId);
         VertexAttributes attributes;
 
         attributes.radius = vertexScalingFactor * 0.03;
@@ -229,8 +229,8 @@ void LocalAlignmentGraph::writeSvg(
         attributes.color = "midnightblue";
 
         attributes.tooltip =
-            OrientedReadId(vertex0.orientedReadId).getString() + " " +
-            OrientedReadId(vertex1.orientedReadId).getString();
+            OrientedReadId::fromValue(vertex0.orientedReadId).getString() + " " +
+            OrientedReadId::fromValue(vertex1.orientedReadId).getString();
 
         edgeAttributes.insert(make_pair(e, attributes));
     }
