@@ -6,7 +6,6 @@
 // Shasta.
 #include "SHASTA_ASSERT.hpp"
 #include "filesystem.hpp"
-#include "MarkerInterval.hpp"
 #include "MurmurHash2.hpp"
 #include "touchMemory.hpp"
 
@@ -151,6 +150,13 @@ public:
     // Rename the supporting memory mapped file, if any.
     void rename(const string& newFileName);
 
+    uint64_t getPageSize() const
+    {
+        SHASTA_ASSERT(isOpen);
+        SHASTA_ASSERT(header);
+        return header->pageSize;
+    }
+
 private:
 
 
