@@ -1,20 +1,28 @@
 #ifdef SHASTA_PYTHON_API
 
 // Shasta.
+#include "AssembledSegment.hpp"
 #include "Assembler.hpp"
+#include "AssemblerOptions.hpp"
+#include "AssemblyGraph.hpp"
 #include "Base.hpp"
 #include "CompactUndirectedGraph.hpp"
 #include "compressAlignment.hpp"
 #include "deduplicate.hpp"
 #include "dset64Test.hpp"
+#include "shastaLapack.hpp"
 #include "LongBaseSequence.hpp"
 #include "mappedCopy.hpp"
+#include "Map.hpp"
+#include "MedianConsensusCaller.hpp"
+#include "MemoryMappedAllocator.hpp"
 #include "MultithreadedObject.hpp"
+#include "Reads.hpp"
 #include "ShortBaseSequence.hpp"
 #include "splitRange.hpp"
 #include "testSpoa.hpp"
+#include "shortestPathBoundedDistance.hpp"
 #include "SimpleBayesianConsensusCaller.hpp"
-#include "MedianConsensusCaller.hpp"
 using namespace shasta;
 
 // Pybind11
@@ -28,23 +36,6 @@ using namespace pybind11;
 PYBIND11_MODULE(shasta, module)
 {
 
-    {
-        // Class used by Assembler::getGlobalMarkerGraphEdgeInformation.
-        using Info = Assembler::GlobalMarkerGraphEdgeInformation;
-        class_<Info>(module, "GlobalMarkerGraphEdgeInformation")
-            .def_readwrite("readId", &Info::readId)
-            .def_readwrite("strand", &Info::strand)
-            .def_readwrite("ordinal0", &Info::ordinal0)
-            .def_readwrite("ordinal1", &Info::ordinal1)
-            .def_readwrite("position0", &Info::position0)
-            .def_readwrite("position1", &Info::position1)
-            .def_readwrite("overlappingBaseCount", &Info::overlappingBaseCount)
-            .def_readwrite("sequence", &Info::sequence)
-            ;
-    }
-
-
-
     // Expose class OrientedReadPair to Python.
     class_<OrientedReadPair>(module, "OrientedReadPair")
         .def_readonly("readIds", &OrientedReadPair::readIds)
@@ -77,8 +68,44 @@ PYBIND11_MODULE(shasta, module)
             arg("readId"),
             arg("strand"),
             arg("fileName"))
+        .def("getReadId",
+            (
+                ReadId (Reads::*)
+                (const string&) const
+            )
+            &Reads::getReadId,
+            "Find the ReadId corresponding to a given read name.")
         ;
 
+
+
+    // Expose class AlignOptions to Python.
+    class_<AlignOptions>(module, "AlignOptions")
+        .def(pybind11::init<>())
+        .def_readwrite("alignMethod", &AlignOptions::alignMethod)
+        .def_readwrite("maxSkip", &AlignOptions::maxSkip)
+        .def_readwrite("maxDrift", &AlignOptions::maxDrift)
+        .def_readwrite("maxTrim", &AlignOptions::maxTrim)
+        .def_readwrite("maxMarkerFrequency", &AlignOptions::maxMarkerFrequency)
+        .def_readwrite("minAlignedMarkerCount", &AlignOptions::minAlignedMarkerCount)
+        .def_readwrite("minAlignedFraction", &AlignOptions::minAlignedFraction)
+        .def_readwrite("matchScore", &AlignOptions::matchScore)
+        .def_readwrite("mismatchScore", &AlignOptions::mismatchScore)
+        .def_readwrite("gapScore", &AlignOptions::gapScore)
+        .def_readwrite("downsamplingFactor", &AlignOptions::downsamplingFactor)
+        .def_readwrite("bandExtend", &AlignOptions::bandExtend)
+        .def_readwrite("maxBand", &AlignOptions::maxBand)
+        .def_readwrite("sameChannelReadAlignmentSuppressDeltaThreshold",
+            &AlignOptions::sameChannelReadAlignmentSuppressDeltaThreshold)
+        .def_readwrite("suppressContainments", &AlignOptions::suppressContainments)
+        .def_readwrite("align4DeltaX", &AlignOptions::align4DeltaX)
+        .def_readwrite("align4DeltaY", &AlignOptions::align4DeltaY)
+        .def_readwrite("align4MinEntryCountPerCell", &AlignOptions::align4MinEntryCountPerCell)
+        .def_readwrite("align4MaxDistanceFromBoundary", &AlignOptions::align4MaxDistanceFromBoundary)
+        ;
+
+
+
     // Expose class Assembler to Python.
     class_<Assembler>(module, "Assembler")
 
@@ -123,6 +150,13 @@ PYBIND11_MODULE(shasta, module)
             arg("seed") = 231,
             arg("enrichmentThreshold"),
             arg("threadCount") = 0)
+        .def("selectKmers4",
+            &Assembler::selectKmers4,
+            arg("k"),
+            arg("markerDensity"),
+            arg("seed") = 231,
+            arg("distanceThreshold"),
+            arg("threadCount") = 0)
 
 
 
@@ -147,7 +181,11 @@ PYBIND11_MODULE(shasta, module)
             &Assembler::getMarkers)
         .def("writeMarkerFrequency",
             &Assembler::writeMarkerFrequency)
-
+        .def("computeSortedMarkers",
+            &Assembler::computeSortedMarkers,
+            arg("threadCount") = 0)
+        .def("accessSortedMarkers",
+            &Assembler::accessSortedMarkers)
 
 
         // Alignment candidates.
@@ -174,6 +212,8 @@ PYBIND11_MODULE(shasta, module)
             arg("threadCount") = 0)
         .def("accessAlignmentCandidates",
             &Assembler::accessAlignmentCandidates)
+        .def("accessAlignmentCandidateTable",
+            &Assembler::accessAlignmentCandidateTable)
         .def("getAlignmentCandidates",
             &Assembler::getAlignmentCandidates)
         .def("writeOverlappingReads",
@@ -190,11 +230,24 @@ PYBIND11_MODULE(shasta, module)
             arg("alignedFractionThreshold"),
             arg("nearDiagonalFractionThreshold"),
             arg("deltaThreshold"),
+            arg("writeTocsv") = false,
             arg("threadCount") = 0)
 
         // Alignments.
         .def("writeAlignmentCandidates",
-            &Assembler::writeAlignmentCandidates)
+             &Assembler::writeAlignmentCandidates,
+             arg("useReadName") = false,
+             arg("verbose") = false)
+        .def("writeAlignmentDetails",
+             &Assembler::writeAlignmentDetails)
+        .def("writeLocalAlignmentCandidateReads",
+             &Assembler::writeLocalAlignmentCandidateReads,
+             arg("readId"),
+             arg("strand"),
+             arg("maxDistance"),
+             arg("allowChimericReads"),
+             arg("allowCrossStrandEdges"),
+             arg("allowInconsistentAlignmentEdges"))
         .def("alignOrientedReads",
             (
                 void (Assembler::*)
@@ -224,16 +277,26 @@ PYBIND11_MODULE(shasta, module)
         .def("alignOrientedReads4",
             (
                 void (Assembler::*)
-                (ReadId, Strand, ReadId, Strand, uint64_t, uint64_t, uint64_t, int64_t, int64_t, int64_t) const
+                (ReadId, Strand, ReadId, Strand,
+                    uint64_t, uint64_t, uint64_t, uint64_t,
+                    uint64_t, double, uint64_t, uint64_t, uint64_t, uint64_t,
+                    int64_t, int64_t, int64_t) const
             )
             &Assembler::alignOrientedReads4,
             arg("readId0"),
             arg("strand0"),
             arg("readId1"),
             arg("strand1"),
-            arg("m"),
             arg("deltaX"),
             arg("deltaY"),
+            arg("minEntryCountPerCell"),
+            arg("maxDistanceFromBoundary"),
+            arg("minAlignedMarkerCount"),
+            arg("minAlignedFraction"),
+            arg("maxSkip"),
+            arg("maxDrift"),
+            arg("maxTrim"),
+            arg("maxBand"),
             arg("matchScore"),
             arg("mismatchScore"),
             arg("gapScore"))
@@ -254,27 +317,13 @@ PYBIND11_MODULE(shasta, module)
 
         // Compute an alignment for each alignment candidate.
         .def("computeAlignments",
-            &Assembler::computeAlignments,
-            arg("alignmentMethod") = 0,
-            arg("maxMarkerFrequency"),
-            arg("maxSkip"),
-            arg("maxDrift"),
-            arg("minAlignedMarkerCount"),
-            arg("minAlignedFraction"),
-            arg("maxTrim"),
-            arg("matchScore"),
-            arg("mismatchScore"),
-            arg("gapScore"),
-            arg("downsamplingFactor"),
-            arg("bandExtend"),
-            arg("maxBand"),
-            arg("suppressContainments"),
-            arg("storeAlignments"),
-            arg("threadCount") = 0)
+            &Assembler::computeAlignments)
         .def("accessCompressedAlignments",
             &Assembler::accessCompressedAlignments)
         .def("accessAlignmentData",
             &Assembler::accessAlignmentData)
+        .def("accessAlignmentDataReadWrite",
+            &Assembler::accessAlignmentDataReadWrite)
         .def("analyzeAlignmentMatrix",
             &Assembler::analyzeAlignmentMatrix,
             arg("readId0"),
@@ -307,28 +356,34 @@ PYBIND11_MODULE(shasta, module)
              arg("maxAlignmentCount"),
              arg("threadCount") = 1
              )
+         .def("createReadGraphMode1",
+              &Assembler::createReadGraphMode1,
+              arg("maxAlignmentCount")
+              )
         .def("accessReadGraph",
             &Assembler::accessReadGraph)
         .def("accessReadGraphReadWrite",
             &Assembler::accessReadGraphReadWrite)
-        .def("flagCrossStrandReadGraphEdges",
-            &Assembler::flagCrossStrandReadGraphEdges,
+        .def("flagCrossStrandReadGraphEdges1",
+            &Assembler::flagCrossStrandReadGraphEdges1,
             arg("maxDistance"),
             arg("threadCount") = 0)
+        .def("flagCrossStrandReadGraphEdges2",
+            &Assembler::flagCrossStrandReadGraphEdges2)
         .def("flagChimericReads",
              &Assembler::flagChimericReads,
             arg("maxChimericReadDistance"),
             arg("threadCount") = 0)
         .def("computeReadGraphConnectedComponents",
-            &Assembler::computeReadGraphConnectedComponents,
-            arg("minComponentSize"))
+            &Assembler::computeReadGraphConnectedComponents)
         .def("writeLocalReadGraphReads",
             &Assembler::writeLocalReadGraphReads,
             arg("readId"),
             arg("strand"),
             arg("maxDistance"),
             arg("allowChimericReads"),
-            arg("allowCrossStrandEdges"))
+            arg("allowCrossStrandEdges"),
+            arg("allowInconsistentAlignmentEdges"))
         .def("removeReadGraphBridges",
              &Assembler::removeReadGraphBridges,
              arg("maxDistance"))
@@ -336,25 +391,25 @@ PYBIND11_MODULE(shasta, module)
              &Assembler::analyzeReadGraph)
         .def("readGraphClustering",
              &Assembler::readGraphClustering)
+        .def("writeReadGraphEdges",
+             &Assembler::writeReadGraphEdges,
+             arg("useReadName") = false)
+        .def("flagInconsistentAlignments",
+             &Assembler::flagInconsistentAlignments,
+             arg("triangleErrorThreshold"),
+             arg("leastSquareErrorThreshold"),
+             arg("leastSquareMaxDistance"),
+             arg("threadCount") = 0)
 
 
 
         // Global marker graph.
         .def("createMarkerGraphVertices",
             &Assembler::createMarkerGraphVertices,
-            arg("alignMethod"),
-            arg("maxMarkerFrequency"),
-            arg("maxSkip"),
-            arg("maxDrift"),
-            arg("matchScore"),
-            arg("mismatchScore"),
-            arg("gapScore"),
-            arg("downsamplingFactor"),
-            arg("bandExtend"),
-            arg("maxBand"),
             arg("minCoverage"),
             arg("maxCoverage"),
             arg("minCoveragePerStrand"),
+            arg("allowDuplicateMarkers"),
             arg("peakFinderMinAreaFraction"),
             arg("peakFinderAreaStartIndex"),
             arg("threadCount") = 0)
@@ -377,29 +432,12 @@ PYBIND11_MODULE(shasta, module)
             )
             &Assembler::getGlobalMarkerGraphVertexMarkers,
             arg("vertexId"))
-        .def("getGlobalMarkerGraphVertexChildren",
-            (
-                vector<MarkerGraph::VertexId> (Assembler::*)
-                (MarkerGraph::VertexId) const
-            )
-            &Assembler::getGlobalMarkerGraphVertexChildren,
-            arg("vertexId"))
-        .def("getGlobalMarkerGraphVertexParents",
-            (
-                vector<MarkerGraph::VertexId> (Assembler::*)
-                (MarkerGraph::VertexId) const
-            )
-            &Assembler::getGlobalMarkerGraphVertexParents,
-            arg("vertexId"))
-        .def("getGlobalMarkerGraphEdgeInformation",
-                &Assembler::getGlobalMarkerGraphEdgeInformation,
-                arg("vertexId0"),
-                arg("vertexId1"))
         .def("findMarkerGraphReverseComplementVertices",
             &Assembler::findMarkerGraphReverseComplementVertices,
             arg("threadCount") = 0)
         .def("accessMarkerGraphReverseComplementVertex",
-            &Assembler::accessMarkerGraphReverseComplementVertex)
+            &Assembler::accessMarkerGraphReverseComplementVertex,
+            arg("readWriteAccess") = false)
         .def("findMarkerGraphReverseComplementEdges",
             &Assembler::findMarkerGraphReverseComplementEdges,
             arg("threadCount") = 0)
@@ -414,16 +452,43 @@ PYBIND11_MODULE(shasta, module)
             &Assembler::refineMarkerGraph,
             arg("refineThreshold"),
             arg("threadCount") = 0)
-
-
+        .def("writeBadMarkerGraphVertices",
+            &Assembler::writeBadMarkerGraphVertices)
+        .def("cleanupDuplicateMarkers",
+            &Assembler::cleanupDuplicateMarkers,
+            arg("threadCount") = 0,
+            arg("minCoverage"),
+            arg("minCoveragePerStrand"),
+            arg("duplicateMarkersPattern1Threshold"),
+            arg("pattern1CreateNewVertices"),
+            arg("pattern2CreateNewVertices"))
+        .def("getMarkerGraphMinCoverageUsed",
+            &Assembler::getMarkerGraphMinCoverageUsed)
+        .def("vertexCoverageStatisticsByKmerId",
+            &Assembler::vertexCoverageStatisticsByKmerId)
+        .def("analyzeMarkerGraphForks",
+            &Assembler::analyzeMarkerGraphForks)
 
         // Edges of the global marker graph.
         .def("createMarkerGraphEdges",
             &Assembler::createMarkerGraphEdges,
             arg("threadCount") = 0)
+        .def("createMarkerGraphEdgesStrict",
+            &Assembler::createMarkerGraphEdgesStrict,
+            arg("minEdgeCoverage"),
+            arg("minEdgeCoveragePerStrand"),
+            arg("threadCount") = 0)
+        .def("createMarkerGraphSecondaryEdges",
+            (
+                void (Assembler::*) (uint32_t, size_t)
+            )
+            &Assembler::createMarkerGraphSecondaryEdges,
+            arg("secondaryEdgeMaxSkip"),
+            arg("threadCount") = 0)
         .def("accessMarkerGraphEdges",
             &Assembler::accessMarkerGraphEdges,
-            arg("accessEdgesReadWrite") = false)
+            arg("accessEdgesReadWrite") = false,
+            arg("accessConnectivityReadWrite") = false)
             .def("transitiveReduction",
             &Assembler::transitiveReduction,
             arg("lowCoverageThreshold"),
@@ -454,13 +519,23 @@ PYBIND11_MODULE(shasta, module)
             &Assembler::assembleMarkerGraphEdges,
             arg("threadCount") = 0,
             arg("markerGraphEdgeLengthThresholdForConsensus"),
-            arg("storeCoverageData"))
+            arg("storeCoverageData"),
+            arg("assembleAllEdges"))
         .def("accessMarkerGraphConsensus",
             &Assembler::accessMarkerGraphConsensus)
         .def("accessMarkerGraphCoverageData",
             &Assembler::accessMarkerGraphCoverageData)
-
-
+        .def("setMarkerGraphEdgeFlags",
+            &Assembler::setMarkerGraphEdgeFlags,
+            arg("wasRemovedByTransitiveReduction"),
+            arg("wasPruned"),
+            arg("isSuperBubbleEdge"),
+            arg("isLowCoverageCrossEdge"),
+            arg("wasAssembled"))
+        .def("writeParallelMarkerGraphEdges",
+            &Assembler::writeParallelMarkerGraphEdges)
+        .def("removeMarkerGraph",
+            &Assembler::removeMarkerGraph)
 
         // Assembly graph.
         .def("createAssemblyGraphEdges",
@@ -475,8 +550,6 @@ PYBIND11_MODULE(shasta, module)
             &Assembler::accessAssemblyGraphOrientedReadsByEdge)
         .def("accessAssemblyGraphVertices",
             &Assembler::accessAssemblyGraphVertices)
-        .def("findAssemblyGraphBubbles",
-            &Assembler::findAssemblyGraphBubbles)
         .def("writeAssemblyGraph",
             &Assembler::writeAssemblyGraph)
         .def("assemble",
@@ -542,6 +615,28 @@ PYBIND11_MODULE(shasta, module)
             arg("offDiagonalRatio"))
         .def("alignPseudoPaths",
             &Assembler::alignPseudoPaths)
+        .def("analyzeAssemblyGraphBubbles",
+            &Assembler::analyzeAssemblyGraphBubbles,
+            arg("debug") = true)
+        .def("removeAssemblyGraph",
+            &Assembler::removeAssemblyGraph)
+
+
+
+        // Assembly mode 2.
+        .def("createAssemblyGraph2",
+            &Assembler::createAssemblyGraph2,
+            arg("bubbleRemovalDiscordantRatioThreshold"),
+            arg("bubbleRemovalAmbiguityThreshold"),
+            arg("bubbleRemovalMaxPeriod"),
+            arg("superbubbleRemovalEdgeLengthThreshold"),
+            arg("phasingMinReadCount"),
+            arg("suppressGfaOutput"),
+            arg("suppressFastaOutput"),
+            arg("suppressDetailedOutput"),
+            arg("suppressPhasedOutput"),
+            arg("suppressHaploidOutput"),
+            arg("threadCount") = 0)
 
 
 
@@ -557,6 +652,7 @@ PYBIND11_MODULE(shasta, module)
             arg("gfaId"))
 
 
+
         .def("test", &Assembler::test)
 
         // Definition of class_Assembler ends here.
@@ -642,6 +738,18 @@ PYBIND11_MODULE(shasta, module)
     module.def("testAlignmentCompression",
         testAlignmentCompression
         );
+    module.def("testMap",
+        testMap
+        );
+    module.def("testMemoryMappedAllocator",
+        MemoryMapped::testMemoryMappedAllocator
+        );
+    module.def("testLapack",
+        testLapack
+        );
+    module.def("testShortestPathBoundedDistance",
+        testShortestPathBoundedDistance
+        );
 }
 
 #endif
