@@ -7,6 +7,8 @@ using namespace shasta;
 // Standard library.
 #include "chrono.hpp"
 #include "iterator.hpp"
+#include <filesystem>
+#include "tuple.hpp"
 
 
 // Load reads from a fastq or fasta file.
@@ -17,6 +19,11 @@ ReadLoader::ReadLoader(
     size_t threadCount,
     const string& dataNamePrefix,
     size_t pageSize,
+    bool detectPalindromesOnFastqLoad,
+    double qScoreRelativeMeanDifference,
+    double qScoreMinimumMean,
+    double qScoreMinimumVariance,
+    bool writePalindromesToCsv,
     Reads& reads):
     
     MultithreadedObject(*this),
@@ -26,6 +33,11 @@ ReadLoader::ReadLoader(
     threadCount(threadCount),
     dataNamePrefix(dataNamePrefix),
     pageSize(pageSize),
+    detectPalindromesOnFastqLoad(detectPalindromesOnFastqLoad),
+    qScoreRelativeMeanDifference(qScoreRelativeMeanDifference),
+    qScoreMinimumMean(qScoreMinimumMean),
+    qScoreMinimumVariance(qScoreMinimumVariance),
+    writePalindromesToCsv(writePalindromesToCsv),
     reads(reads)
 {
     cout << timestamp << "Loading reads from " << fileName << endl;
@@ -78,7 +90,7 @@ void ReadLoader::processFastaFile()
 
     // Read the entire fasta file.
     const auto t0 = std::chrono::steady_clock::now();
-    readFile();
+    allocateBufferAndReadFile();
 
     // Each thread stores reads in its own data structures.
     const auto t1 = std::chrono::steady_clock::now();
@@ -289,7 +301,7 @@ void ReadLoader::processFastqFile()
 
     // Read the entire fastq file.
     const auto t0 = std::chrono::steady_clock::now();
-    readFile();
+    allocateBufferAndReadFile();
 
     // Find all line ends in the file.
     const auto t1 = std::chrono::steady_clock::now();
@@ -324,6 +336,20 @@ void ReadLoader::processFastqFile()
         "Parse: " << seconds(t3-t2) << " s.\n"
         "Store: " << seconds(t4-t3) << " s.\n"
         "Total: " << seconds(t4-t0) << " s." << endl;
+
+
+    // Write a csv file with the list of palindromic reads
+    // if shasta is running in "filterReads" mode.
+    // This should not too big as the typical rate of
+    // palindromic reads is around 1e-4.
+    if(writePalindromesToCsv) {
+        ofstream csvOut("PalindromicReads.csv");
+        for (const auto& item: threadPalindromicReadNames) {
+            for (const auto& name: item) {
+                csvOut << name << "\n";
+            }
+        }
+    }
 }
 
 
@@ -339,6 +365,7 @@ void ReadLoader::processFastqFileThreadFunction(size_t threadId)
     LongBaseSequences& thisThreadReads = *threadReads[threadId];
     MemoryMapped::VectorOfVectors<uint8_t, uint64_t>& thisThreadReadRepeatCounts =
         *threadReadRepeatCounts[threadId];
+    vector<string>& thisThreadPalindromicReadNames = threadPalindromicReadNames[threadId];
 
     // Find the total number of reads in the file.
     SHASTA_ASSERT((lineEnds.size() % 4) == 0); // We already checked for that.
@@ -366,7 +393,7 @@ void ReadLoader::processFastqFileThreadFunction(size_t threadId)
         const auto thisReadLineEnds = lineEnds.begin() + i * 4;
 
         // Locate the header line for this read.
-        const auto headerBegin = fileBegin + ((i==0) ? 0 : (1 + *(thisReadLineEnds-1)));
+        const auto headerBegin = fileBegin + ((i == 0) ? 0 : (1 + *(thisReadLineEnds - 1)));
         const auto headerEnd = fileBegin + thisReadLineEnds[0];
         SHASTA_ASSERT(headerEnd > headerBegin);
 
@@ -386,13 +413,13 @@ void ReadLoader::processFastqFileThreadFunction(size_t threadId)
         SHASTA_ASSERT(scoresEnd > scoresBegin);
 
         // Check the header line.
-        if(headerEnd == headerBegin) {
+        if (headerEnd == headerBegin) {
             throw runtime_error("Empty header line for read at offset " +
-                to_string(headerBegin-fileBegin) + ".");
+                to_string(headerBegin - fileBegin) + ".");
         }
-        if(*headerBegin != '@') {
+        if (*headerBegin != '@') {
             throw runtime_error("Read at offset " +
-                to_string(headerBegin-fileBegin) +
+                to_string(headerBegin - fileBegin) +
                 " does not begin with \"@\".");
         }
 
@@ -401,24 +428,24 @@ void ReadLoader::processFastqFileThreadFunction(size_t threadId)
         // first white space.
         readName.clear();
         const auto nameBegin = headerBegin + 1;
-        for(auto it=nameBegin; it!=headerEnd; ++it) {
+        for (auto it = nameBegin; it != headerEnd; ++it) {
             const char c = *it;
-            if(std::isspace(c)) {
+            if (std::isspace(c)) {
                 break;
             }
             readName.push_back(c);
         }
         if(readName.empty()) {
             throw runtime_error("Empty name for read at offset " +
-                to_string(headerBegin-fileBegin) + ".");
+                to_string(headerBegin - fileBegin) + ".");
         }
 
         // Extract the read meta data. It starts at the first non-space character
         // following the read name.
         readMetaData.clear();
-        for(auto it = nameBegin + readName.size(); it != headerEnd; ++it) {
+        for (auto it = nameBegin + readName.size(); it != headerEnd; ++it) {
             const char c = *it;
-            if(isspace(c) and readMetaData.empty()) {
+            if (isspace(c) and readMetaData.empty()) {
                 // Do nothing. Only start storing at the first non-space character.
             } else {
                 readMetaData.push_back(c);
@@ -427,46 +454,64 @@ void ReadLoader::processFastqFileThreadFunction(size_t threadId)
 
 
         // Check the line containing the plus.
-        if(plusEnd - plusBegin != 1) {
-            throw runtime_error("Extraneous characters on third line for read " +
-                readName + " at offset " + to_string(headerBegin-fileBegin) + ".");
-        }
-        if(*plusBegin != '+') {
+        // It can contain arbitrary characters after the plus.
+        // We already checked above that it is at least 1 character long.
+        if (*plusBegin != '+') {
             throw runtime_error("Third line does not contain \"+\" for read " +
-                readName + " at offset " + to_string(headerBegin-fileBegin) + ".");
+                                readName + " at offset " + to_string(headerBegin - fileBegin) + ".");
         }
 
         // Get the number of bases.
         const auto baseCount = sequenceEnd - sequenceBegin;
-        if(scoresEnd - scoresBegin != baseCount) {
+        if (scoresEnd - scoresBegin != baseCount) {
             throw runtime_error(
-                "Inconsistent numbers of bases and quality scores for read " +
-                readName + " at offset " +
-                to_string(headerBegin-fileBegin) + ": " +
-                to_string(baseCount) + " bases, " +
-                to_string(scoresEnd - scoresBegin) + " quality scores."
-                );
+                    "Inconsistent numbers of bases and quality scores for read " +
+                    readName + " at offset " +
+                    to_string(headerBegin - fileBegin) + ": " +
+                    to_string(baseCount) + " bases, " +
+                    to_string(scoresEnd - scoresBegin) + " quality scores."
+            );
         }
 
         // Get the bases.
         read.clear();
-        for(auto it=sequenceBegin; it!=sequenceEnd; ++it) {
+        for (auto it = sequenceBegin; it != sequenceEnd; ++it) {
             const char c = *it;
             const Base base = Base::fromCharacterNoException(c);
-            if(!base.isValid()) {
+            if (!base.isValid()) {
                 throw runtime_error("Invalid base " + string(1, c) + " for read " +
-                    readName + " at offset " + to_string(it-fileBegin) + ".");
+                                    readName + " at offset " + to_string(it - fileBegin) + ".");
             }
             read.push_back(base);
         }
 
         // If the read is too short, skip it.
-        if(read.size() < minReadLength) {
+        if (read.size() < minReadLength) {
             __sync_fetch_and_add(&discardedShortReadReadCount, 1);
             __sync_fetch_and_add(&discardedShortReadBaseCount, read.size());
             continue;
         }
 
+        // Skip if the q scores have an obvious palindromic characteristic
+        if (detectPalindromesOnFastqLoad){
+            span<char> scores(scoresBegin, scoresEnd);
+            bool isPalindrome = isPalindromic(
+                    scores,
+                    qScoreRelativeMeanDifference,
+                    qScoreMinimumMean,
+                    qScoreMinimumVariance);
+
+            if (isPalindrome) {
+                __sync_fetch_and_add(&discardedPalindromicReadCount, 1);
+                __sync_fetch_and_add(&discardedPalindromicBaseCount, read.size());
+
+                // Update the running list of palindromic read names for this thread
+                thisThreadPalindromicReadNames.push_back(readName);
+
+                continue;
+            }
+        }
+
         // Store the read.
         if(computeRunLengthRepresentation(read, runLengthRead, readRepeatCount)) {
             thisThreadReadNames.appendVector(readName.begin(), readName.end());
@@ -480,85 +525,96 @@ void ReadLoader::processFastqFileThreadFunction(size_t threadId)
     }
 }
 
-#ifdef __linux__
-int ReadLoader::tryDirectIO(const string& fileName) {
-    auto fd = ::open(fileName.c_str(), O_RDONLY | O_DIRECT);
-    if (fd == -1) {
-        // This could happen for a variety of reasons. If the file is readable and it still cannot
-        // be opened, it is likely that the volume/file-system does not support the O_DIRECT flag.
-        return O_RDONLY;
-    }
-    
-    // Verify that the O_DIRECT flag is actually supported. Sometimes the error shows up in the read call.
-    MemoryMapped::Vector<char> tempBuffer;
-    tempBuffer.createNew("tmp-odirectReadTestBuffer", pageSize, pageSize);
-    auto bytesRead = ::read(fd, &tempBuffer[0], pageSize);
-    ::close(fd);
-    tempBuffer.remove();
-    if (bytesRead == -1) {
-        return O_RDONLY;
-    }
 
-    // File can be opened and read using Direct IO.
-    return O_RDONLY | O_DIRECT;
-}
-#endif
 
-// Read an entire file into a buffer,
-// using threadCountForReading threads.
-void ReadLoader::readFile()
+void ReadLoader::allocateBufferAndReadFile()
 {
-    // Create a buffer to contain the entire file.
-    const auto t0 = std::chrono::steady_clock::now();
-    int64_t bytesToRead = filesystem::fileSize(fileName);
-    buffer.createNew(dataName("tmp-FastaBuffer"), pageSize);
-    // Do reserve before resize, to force using exactly the
-    // amount of memory necessary and nothing more.
-    buffer.reserve(bytesToRead);
-    buffer.resize(bytesToRead);
+    allocateBuffer();
 
-    // Open the input file.
-    int flags = O_RDONLY;
+    // Try reading using the requested setting of noCache/O_DIRECT.
+    // If successful, all done.
+    if(readFile(noCache)) {
+        return;
+    }
+
+    // If there was failure and we are using noCache, try turning it off.
+    // If successful, all done.
 #ifdef __linux__
     if(noCache) {
-        flags = tryDirectIO(fileName);
-        if (flags != (O_RDONLY | O_DIRECT)) {
-            cout << "--Reads.noCache was turned off for " << fileName
-                << " because it is not supported by the filesystem." << endl;
+        cout << "Turning off --Reads.noCache for " << fileName << endl;
+        if(readFile(false)) {
+            return;
         }
     }
 #endif
+
+    // If getting here, nothing worked.
+    throw runtime_error("Error reading " + fileName);
+}
+
+
+
+void ReadLoader::allocateBuffer()
+{
+    const auto t0 = std::chrono::steady_clock::now();
+
+    // Create a buffer to contain the entire file.
+    fileSize = std::filesystem::file_size(fileName);
+    buffer.createNew(dataName("tmp-FastaBuffer"), pageSize);
+
+    // Do reserve before resize, to force using exactly the
+    // amount of memory necessary and nothing more.
+    buffer.reserve(fileSize);
+    buffer.resize(fileSize);
+
+    const auto t1 = std::chrono::steady_clock::now();
+
+    cout <<  "File size: " << fileSize << " bytes." << endl;
+    cout << "Buffer allocate time: " << seconds(t1 - t0) << " s." << endl;
+}
+
+
+
+// Read an entire file into the buffer.
+bool ReadLoader::readFile(bool useODirect)
+{
+    const auto t0 = std::chrono::steady_clock::now();
+
+    // Set up flags to open the file.
+    int flags = O_RDONLY;
+#ifdef __linux__
+    if(useODirect) {
+        flags |= O_DIRECT;
+    }
+#endif
+
+    // Open the input file.
     const int fileDescriptor = ::open(fileName.c_str(), flags);
     if(fileDescriptor == -1) {
-        throw runtime_error("Error opening " + fileName + " for read.");
+        return false;
     }
 
     // Read it in.
-    const auto t1 = std::chrono::steady_clock::now();
     char* bufferPointer = &buffer[0];
     uint64_t bufferCapacity = buffer.capacity();
+    uint64_t bytesToRead = fileSize;
     while(bytesToRead) {
         const int64_t bytesRead = ::read(fileDescriptor, bufferPointer, bufferCapacity);
         if(bytesRead == -1) {
             ::close(fileDescriptor);
-            throw runtime_error("Error reading from " + fileName + " near offset " +
-                to_string(buffer.size()-bytesToRead));
+            return false;
         }
         bufferPointer += bytesRead;
         bytesToRead -= bytesRead;
         bufferCapacity -= bytesRead;
     }
     ::close(fileDescriptor);
-    const auto t2 = std::chrono::steady_clock::now();
+    const auto t1 = std::chrono::steady_clock::now();
     const double t01 = seconds(t1 - t0);
-    const double t12 = seconds(t2 - t1);
-
-    cout <<  "File size: " << buffer.size() << " bytes." << endl;
-    cout << "Allocate buffer time: " << t01 << " s." << endl;
-    cout << "Read time: " << t12 << " s." << endl;
-    cout << "Read rate: " << double(buffer.size()) / t12 << " bytes/s." << endl;
-
 
+    cout << "Read time: " << t01 << " s." << endl;
+    cout << "Read rate: " << double(fileSize) / t01 << " bytes/s." << endl;
+    return true;
 }
 
 
@@ -634,7 +690,10 @@ void ReadLoader::allocatePerThreadDataStructures()
     threadReadMetaData.resize(threadCount);
     threadReads.resize(threadCount);
     threadReadRepeatCounts.resize(threadCount);
+    threadPalindromicReadNames.resize(threadCount);
 }
+
+
 void ReadLoader::allocatePerThreadDataStructures(size_t threadId)
 {
     threadReadNames[threadId] = make_unique< MemoryMapped::VectorOfVectors<char, uint64_t> >();
