@@ -2,6 +2,7 @@
 #define SHASTA_READ_LOADER_HPP
 
 // shasta
+#include "PalindromeQuality.hpp"
 #include "LongBaseSequence.hpp"
 #include "MemoryMappedObject.hpp"
 #include "MultithreadedObject.hpp"
@@ -30,6 +31,11 @@ public:
         size_t threadCount,
         const string& dataNamePrefix,
         size_t pageSize,
+        bool detectPalindromesOnFastqLoad,
+        double qScoreRelativeMeanDifference,
+        double qScoreMinimumMean,
+        double qScoreMinimumVariance,
+        bool writePalindromesToCsv,
         Reads& reads);
 
     ~ReadLoader();
@@ -44,6 +50,11 @@ public:
     uint64_t discardedShortReadReadCount = 0;
     uint64_t discardedShortReadBaseCount = 0;
 
+    // The number of reads and raw bases discarded because the read had
+    // a q score distribution that was indicative of a palindrome.
+    uint64_t discardedPalindromicReadCount = 0;
+    uint64_t discardedPalindromicBaseCount = 0;
+
     // The number of reads and raw bases discarded because the read
     // contained repeat counts greater than 255.
     uint64_t discardedBadRepeatCountReadCount = 0;
@@ -71,9 +82,20 @@ private:
     const string& dataNamePrefix;
     const size_t pageSize;
 
+    // Boolean switch to use quality scores to skip reads that have an indication of palindromic sequence
+    bool detectPalindromesOnFastqLoad;
+
+    // Each of the 3 thresholds necessary for calling isPalindromic()
+    double qScoreRelativeMeanDifference;
+    double qScoreMinimumMean;
+    double qScoreMinimumVariance;
+
+    // This is true if shasta was run with command "filterReads"
+    bool writePalindromesToCsv;
+
     // The data structure that the reads will be added to.
     Reads& reads;
-    
+
     // Create the name to be used for a MemoryMapped object.
     string dataName(
         const string& dataName) const;
@@ -83,8 +105,11 @@ private:
 
     // Read an entire file into a buffer,
     // using threadCountForReading threads.
+    int64_t fileSize;
     MemoryMapped::Vector<char> buffer;
-    void readFile();
+    void allocateBuffer();
+    bool readFile(bool useODirect);
+    void allocateBufferAndReadFile();
 
     // Vectors where each thread stores the reads it found.
     // Indexed by threadId.
@@ -92,6 +117,7 @@ private:
     vector< unique_ptr<MemoryMapped::VectorOfVectors<char, uint64_t> > > threadReadMetaData;
     vector< unique_ptr<LongBaseSequences> > threadReads;
     vector< unique_ptr<MemoryMapped::VectorOfVectors<uint8_t, uint64_t> > > threadReadRepeatCounts;
+    vector< vector<string> > threadPalindromicReadNames;
     void allocatePerThreadDataStructures();
     void allocatePerThreadDataStructures(size_t threadId);
 
@@ -116,9 +142,6 @@ private:
     vector< vector<uint64_t> > threadLineEnds;
     vector<uint64_t> lineEnds;
 
-#ifdef __linux__
-    int tryDirectIO(const string& fileName);
-#endif
 };
 
 
